from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools.BoundedInterval import BoundedInterval
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals


def test_IntervalTree_find_intervals_starting_and_stopping_within_interval_01():
    blocks = _make_test_intervals()
    target_interval = BoundedInterval(-10, 0)
    expected_payloads = ()
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_and_stopping_within_interval(target_interval)
        assert expected_blocks == actual_blocks

def test_IntervalTree_find_intervals_starting_and_stopping_within_interval_02():
    blocks = _make_test_intervals()
    target_interval = BoundedInterval(0, 9)
    expected_payloads = ('a', 'd',)
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_and_stopping_within_interval(target_interval)
        assert expected_blocks == actual_blocks

def test_IntervalTree_find_intervals_starting_and_stopping_within_interval_03():
    blocks = _make_test_intervals()
    target_interval = BoundedInterval(4, 19)
    expected_payloads = ('b', 'c', 'd', 'g',)
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_and_stopping_within_interval(target_interval)
        assert expected_blocks == actual_blocks

def test_IntervalTree_find_intervals_starting_and_stopping_within_interval_04():
    blocks = _make_test_intervals()
    target_interval = BoundedInterval(6, 10)
    expected_payloads = ('c', 'd',)
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_and_stopping_within_interval(target_interval)
        assert expected_blocks == actual_blocks

def test_IntervalTree_find_intervals_starting_and_stopping_within_interval_05():
    blocks = _make_test_intervals()
    target_interval = BoundedInterval(13, 15)
    expected_payloads = ()
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_and_stopping_within_interval(target_interval)
        assert expected_blocks == actual_blocks

def test_IntervalTree_find_intervals_starting_and_stopping_within_interval_06():
    blocks = _make_test_intervals()
    target_interval = BoundedInterval(14, 25)
    expected_payloads = ('e', 'f', 'g', 'h',)
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_and_stopping_within_interval(target_interval)
        assert expected_blocks == actual_blocks

def test_IntervalTree_find_intervals_starting_and_stopping_within_interval_07():
    blocks = _make_test_intervals()
    target_interval = BoundedInterval(19, 26)
    expected_payloads = ('h',)
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_and_stopping_within_interval(target_interval)
        assert expected_blocks == actual_blocks

def test_IntervalTree_find_intervals_starting_and_stopping_within_interval_08():
    blocks = _make_test_intervals()
    target_interval = BoundedInterval(24, 31)
    expected_payloads = ('i', 'j',)
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_and_stopping_within_interval(target_interval)
        assert expected_blocks == actual_blocks

def test_IntervalTree_find_intervals_starting_and_stopping_within_interval_09():
    blocks = _make_test_intervals()
    target_interval = BoundedInterval(26, 29)
    expected_payloads = ('j',)
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_and_stopping_within_interval(target_interval)
        assert expected_blocks == actual_blocks

def test_IntervalTree_find_intervals_starting_and_stopping_within_interval_10():
    blocks = _make_test_intervals()
    target_interval = BoundedInterval(30, 40)
    expected_payloads = ('k', 'l',)
    expected_blocks = tuple(sorted(filter(lambda x: x.keys()[0] in expected_payloads, blocks),
        key = lambda x: x.signature))
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0)) # rotate to permute tree construction
        tree = IntervalTree(blocks)
        actual_blocks = tree.find_intervals_starting_and_stopping_within_interval(target_interval)
        assert expected_blocks == actual_blocks




from abjad.tools.pitchtools._Segment import _Segment


class _IntervalClassSegment(_Segment):
    '''.. versionadded:: 2.0

    Interval-class segment base class.
    '''

    ### OVERLOADS ###

    def __repr__(self):
        return '%s(%s)' % (type(self).__name__, self._format_string)

    ### PRIVATE ATTRIBUTES ###

    @property
    def _format_string(self):
        return ', '.join([str(x) for x in self])

    ### PUBLIC ATTRIBUTES ###

    @property
    def interval_class_numbers(self):
        return tuple([interval_class.number for interval_class in self])

    @property
    def interval_classes(self):
        return tuple(self[:])




from abjad.tools.musicxmltools._all_are_nodes_with_tag import _all_are_nodes_with_tag
from abjad.tools.musicxmltools._get_voice_id_in_note_node import _get_voice_id_in_note_node


def _get_voice_ids_in_note_nodes(nodes):
    assert _all_are_nodes_with_tag(nodes, 'note')
    ids = []
    for node in nodes:
        id = _get_voice_id_in_note_node(node)
        if id is not None and id not in ids:
            ids.append(id)
    return tuple(ids)



from abjad.core import _Immutable
from abjad.tools.schemetools.SchemePair import SchemePair


class SchemeAssociativeList(tuple, _Immutable):
    '''.. versionadded:: 2.0

    Abjad model of Scheme associative list::

        abjad> schemetools.SchemeAssociativeList(('space', 2), ('padding', 0.5))
        SchemeAssociativeList(SchemePair('space', 2), SchemePair('padding', 0.5))

    Scheme associative lists are immutable.
    '''

    def __new__(klass, *args):
        args_as_pairs = []
        for arg in args:
            if not isinstance(arg, (tuple, SchemePair)):
                raise TypeError('must be Python pair or Scheme pair: "%s".' % str(arg))
            arg_as_pair = SchemePair(*arg)
            args_as_pairs.append(arg_as_pair)
        self = tuple.__new__(klass, args_as_pairs)
        return self

    def __getnewargs__(self):
        return tuple(self)

    ### OVERLOADS ###

    def __repr__(self):
        return '%s(%s)' % (type(self).__name__, self._format_string)

    def __str__(self):
        return '(%s)' % self._output_string

    ### PRIVATE ATTRIBUTES ###

    @property
    def _format_string(self):
        return ', '.join([repr(x) for x in self])

    @property
    def _output_string(self):
        vals = []
        for x in self:
            if isinstance(x, type(True)) and x:
                vals.append("#t")
            elif isinstance(x, type(True)):
                vals.append("#f")
            else:
                vals.append(x)
        return ' '.join([str(x) for x in vals])

    ### PUBLIC ATTRIBUTES ###

    @property
    def format(self):
        '''LilyPond input format of Scheme associative list::

            abjad> scheme_associative_list = schemetools.SchemeAssociativeList(('space', 2), ('padding', 0.5))
            abjad> scheme_associative_list.format
            "#'((space . 2) (padding . 0.5))"

        Return string.
        '''
        return "#'%s" % self.__str__()



from abjad.tools import contexttools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools._WindInstrument import _WindInstrument


class Flute(_WindInstrument):
    r'''.. versionadded:: 2.0

    Abjad model of the flute::

        abjad> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        abjad> instrumenttools.Flute()(staff)
        Flute('Flute', 'Fl.')(Staff{4})

    ::

        abjad> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { Flute }
            \set Staff.shortInstrumentName = \markup { Fl. }
            c'8
            d'8
            e'8
            f'8
        }

    The flute targets staff context by default.
    '''

    def __init__(self,
        instrument_name = 'Flute', short_instrument_name = 'Fl.', target_context = None):
        _WindInstrument.__init__(self, instrument_name, short_instrument_name, target_context)
        self.sounding_pitch_of_written_middle_c = pitchtools.NamedChromaticPitch("c'")
        self.primary_clefs = [contexttools.ClefMark('treble')]
        self._copy_primary_clefs_to_all_clefs()
        self.traditional_range = (0, 38)



from abjad.tools.musicxmltools._all_are_nodes_with_tag import _all_are_nodes_with_tag
from abjad.tools.musicxmltools._note_nodes_contain_chords import _note_nodes_contain_chords


def _group_note_nodes_by_chord(nodes):
    assert _all_are_nodes_with_tag(nodes, 'note')
    if not _note_nodes_contain_chords(nodes):
        return [[node] for node in nodes]
    groups = []
    group = [nodes[0]]
    for node in nodes[1:]:
        if node.find('chord') is not None:
            group.append(node)
        else:
            groups.append(group)
            group = [node]
    groups.append(group)
    return groups



from abjad.tools.musicxmltools._all_are_nodes_with_tag import _all_are_nodes_with_tag
from abjad.tools.musicxmltools._get_staff_id_in_note_node import _get_staff_id_in_note_node
from abjad.tools.musicxmltools._get_staff_ids_in_note_nodes import _get_staff_ids_in_note_nodes

def _group_note_nodes_by_staff_id(nodes):
    assert _all_are_nodes_with_tag(nodes, 'note')
    ids = _get_staff_ids_in_note_nodes(nodes)
    if not ids:
        return [nodes]
    else:
        groups = []
        for id in ids:
            groups.append(filter(\
                lambda x: _get_staff_id_in_note_node(x) == id, nodes))
        return groups



from abjad.tools.intervaltreetools import BoundedInterval
from abjad.tools.intervaltreetools import IntervalTree
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals


def test_IntervalTree_bounds_01():
    tree = IntervalTree([])
    assert tree.bounds is None


def test_IntervalTree_bounds_02():
    tree = IntervalTree(_make_test_intervals())
    assert tree.bounds == BoundedInterval(0, 37)



class _BlockAttributed(object):
    '''Model of attribute block in LilyPond input file.
    '''

    def __init__(self):
        self._is_formatted_when_empty = False

    ### OVERLOADS ###

    def __repr__(self):
        if not len(self._user_attributes):
            return '%s()' % type(self).__name__
        else:
            return '%s(%s)' % (type(self).__name__, len(self._user_attributes))

    ### PRIVATE ATTRIBUTES ###

    @property
    def _format_pieces(self):
        result = []
        if not self._formatted_user_attributes and not getattr(self, 'contexts', None):
            if self.is_formatted_when_empty:
                result.append('%s {}' % self._escaped_name)
                return result
            else:
                return result
        result.append('%s {' % self._escaped_name)
        if getattr(self, 'contexts', None):
            specs = self._formatted_context_specifications
            result.extend(['\t' + x for x in specs])
        formatted_attributes = self._formatted_user_attributes
        formatted_attributes = ['\t' + x for x in formatted_attributes]
        result.extend(formatted_attributes)
        result.append('}')
        return result

    @property
    def _formatted_user_attributes(self):
        from abjad.tools.lilyfiletools._format_lilypond_value import _format_lilypond_value
        result = []
        for key, value in sorted(vars(self).items()):
            if not key.startswith('_'):
                formatted_key = key.replace('_', '-')
                formatted_value = _format_lilypond_value(value)
                setting = '%s = %s' % (formatted_key, formatted_value)
                result.append(setting)
        return result

    @property
    def _user_attributes(self):
        all_attributes = vars(self).keys()
        user_attributes = [x for x in all_attributes if not x.startswith('_')]
        user_attributes.sort()
        return user_attributes

    ### PUBLIC ATTRIBUTES ###

    @property
    def format(self):
        return '\n'.join(self._format_pieces)

    @apply
    def is_formatted_when_empty():
        def fget(self):
            return self._is_formatted_when_empty
        def fset(self, arg):
            if isinstance(arg, type(True)):
                self._is_formatted_when_empty = arg
            else:
                raise TypeError
        return property(**locals())



from abjad.tools.contexttools.ContextMark import ContextMark


class InstrumentMark(ContextMark):
    r'''.. versionadded:: 2.0

    Abjad model of an instrument change::

        abjad> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        abjad> contexttools.InstrumentMark('Flute', 'Fl.')(staff)
        InstrumentMark('Flute', 'Fl.')(Staff{4})

    ::

        abjad> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { Flute }
            \set Staff.shortInstrumentName = \markup { Fl. }
            c'8
            d'8
            e'8
            f'8
        }

    Instrument marks target staff context by default.
    '''

    _format_slot = 'opening'

    def __init__(self, instrument_name, short_instrument_name, target_context = None):
        from abjad.tools.stafftools.Staff import Staff
        from abjad.tools.markuptools import Markup
        ContextMark.__init__(self, target_context = target_context)
        if self.target_context is None:
            self._target_context = Staff
        self._instrument_name = Markup(instrument_name)
        self._short_instrument_name = Markup(short_instrument_name)

    ### OVERLOADS ###

    def __copy__(self, *args):
        return type(self)(self._instrument_name, self._short_instrument_name,
            target_context = self.target_context)

    def __eq__(self, arg):
        if isinstance(arg, type(self)):
            if self.instrument_name == arg.instrument_name:
                if self.short_instrument_name == arg.short_instrument_name:
                    return True
        return False

    ### PRIVATE ATTRIBUTES ###

    @property
    def _contents_repr_string(self):
        markups = (self.instrument_name, self.short_instrument_name)
        contents_string = ', '.join([repr(markup._contents_string) for markup in markups])
        return contents_string

    # will probably need to change definition at some point #
    @property
    def _target_context_name(self):
        return self.target_context.__name__

    ### PUBLIC ATTRIBUTES ###

    @property
    def format(self):
        '''Read-only LilyPond input format of instrument mark:

        ::

            abjad> instrument = contexttools.InstrumentMark('Flute', 'Fl.')
            abjad> instrument.format
            ['\\set Staff.instrumentName = \\markup { Flute }', '\\set Staff.shortInstrumentName = \\markup { Fl. }']

        Return list.
        '''
        result = []
        result.append(r'\set %s.instrumentName = %s' % (self._target_context_name, self.instrument_name))
        result.append(r'\set %s.shortInstrumentName = %s' % (self._target_context_name, self.short_instrument_name))
        return result

    @apply
    def instrument_name():
        def fget(self):
            r'''Get instrument name::

                abjad> instrument = contexttools.InstrumentMark('Flute', 'Fl.')
                abjad> instrument.instrument_name
                Markup('Flute')

            Set instrument name::

                abjad> instrument.instrument_name = 'Alto Flute'
                abjad> instrument.instrument_name
                Markup('Alto Flute')

            Return markup.
            '''
            return self._instrument_name
        def fset(self, instrument_name):
            from abjad.tools.markuptools import Markup
            assert isinstance(instrument_name, str)
            self._instrument_name = Markup(instrument_name)
        return property(**locals())

    @apply
    def short_instrument_name():
        def fget(self):
            r'''Get short instrument name::

                abjad> instrument = contexttools.InstrumentMark('Flute', 'Fl.')
                abjad> instrument.short_instrument_name
                Markup('Fl.')

            Set short instrument name::

                abjad> instrument.short_instrument_name = 'Alto Fl.'
                abjad> instrument.short_instrument_name
                Markup('Alto Fl.')

            Return markup.
            '''
            return self._short_instrument_name
        def fset(self, short_instrument_name):
            from abjad.tools.markuptools import Markup
            assert isinstance(short_instrument_name, str)
            self._short_instrument_name = Markup(short_instrument_name)
        return property(**locals())





from abjad import *
from abjad.tools import durationtools
from abjad.tools import verticalitytools


def test_VerticalMoment___len___01():

    score = Score([])
    score.append(Staff([tuplettools.FixedDurationTuplet(Duration(4, 8), notetools.make_repeated_notes(3))]))
    piano_staff = scoretools.PianoStaff([])
    piano_staff.append(Staff(notetools.make_repeated_notes(2, Duration(1, 4))))
    piano_staff.append(Staff(notetools.make_repeated_notes(4)))
    contexttools.ClefMark('bass')(piano_staff[1])
    score.append(piano_staff)
    pitchtools.set_ascending_named_diatonic_pitches_on_nontied_pitched_components_in_expr(list(reversed(score.leaves)))

    r'''
    \new Score <<
        \new Staff {
            \times 4/3 {
                d''8
                c''8
                b'8
            }
        }
        \new PianoStaff <<
            \new Staff {
                a'4
                g'4
            }
            \new Staff {
                \clef "bass"
                f'8
                e'8
                d'8
                c'8
            }
        >>
    >>
    '''

    vertical_moment = verticalitytools.get_vertical_moment_at_prolated_offset_in_expr(score, durationtools.Offset(1, 8))
    "VerticalMoment(Score<<2>>, Staff{1}, {@ 3:4 d''8, c''8, b'8 @}, d''8, PianoStaff<<2>>, Staff{2}, a'4, Staff{4}, e'8)"
    assert len(vertical_moment) == 9

    vertical_moment = verticalitytools.get_vertical_moment_at_prolated_offset_in_expr(score[0], durationtools.Offset(1, 8))
    "VerticalMoment(Staff{1}, {@ 3:4 d''8, c''8, b'8 @}, d''8)"
    assert len(vertical_moment) == 3

    vertical_moment = verticalitytools.get_vertical_moment_at_prolated_offset_in_expr(piano_staff, durationtools.Offset(1, 8))
    "VerticalMoment(PianoStaff<<2>>, Staff{2}, a'4, Staff{4}, e'8)"
    assert len(vertical_moment) == 5

    vertical_moment = verticalitytools.get_vertical_moment_at_prolated_offset_in_expr(piano_staff[0], durationtools.Offset(1, 8))
    "VerticalMoment(Staff{2}, a'4)"
    assert len(vertical_moment) == 2

    vertical_moment = verticalitytools.get_vertical_moment_at_prolated_offset_in_expr(piano_staff[1], durationtools.Offset(1, 8))
    "VerticalMoment(Staff{2}, e'8)"
    assert len(vertical_moment) == 2



from abjad import *


def test_Accidental_alphabetic_string_01():
    t = pitchtools.Accidental('s')
    assert t.alphabetic_string == 's'


def test_Accidental_alphabetic_string_02():
    t = pitchtools.Accidental('')
    assert t.alphabetic_string == ''


def test_Accidental_alphabetic_string_03():
    t = pitchtools.Accidental()
    assert t.alphabetic_string == ''


def test_Accidental_alphabetic_string_04():
    t = pitchtools.Accidental('!')
    assert t.alphabetic_string == '!'




from abjad.core import _Immutable
from abjad.core import _StrictComparator


class Accidental(_StrictComparator, _Immutable):
    '''.. versionadded:: 2.0

    Abjad model of the accidental:

    ::

        abjad> pitchtools.Accidental('s')
        Accidental('s')

    Accidentals are immutable.
    '''

    __slots__ = ('_alphabetic_string', '_is_adjusted', '_name', '_semitones', '_symbolic_string')

    def __new__(klass, arg = ''):
        self = object.__new__(klass)
        # initializer symbolic string from arg
        if arg in self._all_accidental_alphabetic_strings:
            _alphabetic_string = arg
        elif arg in self._all_accidental_symbolic_strings:
            _alphabetic_string = self._symbolic_string_to_alphabetic_string[arg]
        elif arg in self._all_accidental_names:
            _alphabetic_string = self._name_to_alphabetic_string[arg]
        elif arg in self._all_accidental_semitone_values:
            _alphabetic_string = self._semitones_to_alphabetic_string[arg]
        elif isinstance(arg, type(self)):
            _alphabetic_string = arg.alphabetic_string
        elif isinstance(arg, type(None)):
            _alphabetic_string = ''
        else:
            raise ValueError('can not initialize accidental from value: %s' % arg)
        object.__setattr__(self, '_alphabetic_string', _alphabetic_string)
        # initialize derived attributes
        _semitones = self._alphabetic_string_to_semitones[self.alphabetic_string]
        object.__setattr__(self, '_semitones', _semitones)
        _name = self._alphabetic_string_to_name[self.alphabetic_string]
        object.__setattr__(self, '_name', _name)
        _is_adjusted = not self.semitones == 0
        object.__setattr__(self, '_is_adjusted', _is_adjusted)
        _symbolic_string = self._alphabetic_string_to_symbolic_string[self.alphabetic_string]
        object.__setattr__(self, '_symbolic_string', _symbolic_string)
        return self

    def __getnewargs__(self):
        return (self.alphabetic_string,)

    ### OVERLOADS ###

    def __add__(self, arg):
        if not isinstance(arg, type(self)):
            raise TypeError('can only add accidental to other accidental.')
        semitones = self.semitones + arg.semitones
        return type(self)(semitones)

    def __eq__(self, arg):
        if isinstance(arg, type(self)):
            if self.alphabetic_string == arg.alphabetic_string:
                return True
        return False

    def __ge__(self, arg):
        return self.semitones >= arg.semitones

    def __gt__(self, arg):
        return self.semitones > arg.semitones

    def __le__(self, arg):
        return self.semitones <= arg.semitones

    def __lt__(self, arg):
        return self.semitones < arg.semitones

    def __ne__(self, arg):
        return not self == arg

    def __neg__(self):
        return type(self)(-self.semitones)

    def __nonzero__(self):
        return True

    def __repr__(self):
        return "%s('%s')" % (type(self).__name__, self.alphabetic_string)

    def __str__(self):
        return self.alphabetic_string

    def __sub__(self, arg):
        if not isinstance(arg, type(self)):
            raise TypeError('can only sub accidental from other accidental.')
        semitones = self.semitones - arg.semitones
        return type(self)(semitones)

    ### PRIVATE ATTRIBUTES ###

    @property
    def _all_accidental_alphabetic_strings(self):
        return self._alphabetic_string_to_symbolic_string.keys()

    _alphabetic_string_to_name = {
        'ss'  : 'double sharp',
        'tqs' : 'three-quarters sharp',
        's'   : 'sharp',
        'qs'  : 'quarter-sharp',
        ''    : 'natural',
        '!'   : 'forced natural',
        'qf'  : 'quarter-flat',
        'f'   : 'flat',
        'tqf' : 'three-quarters flat',
        'ff'  : 'double flat',
    }

    _alphabetic_string_to_semitones = {
        'ff'  : -2,
        'tqf' : -1.5,
        'f'   : -1,
        'qf'  : -0.5,
        ''    : 0,
        '!'   : 0,
        'qs'  : 0.5,
        's'   : 1,
        'tqs' : 1.5,
        'ss'  : 2,
    }

    _alphabetic_string_to_symbolic_string = {
        'ff': 'bb',
        'tqf': 'b+',
        'f': 'b',
        'qf': 'b-',
        '': '',
        '!': '!',
        'qs': '#-',
        's': '#',
        'tqs': '#+',
        'ss': '###',
    }

    _name_to_alphabetic_string = {
        'double sharp'            : 'ss',
        'three-quarters sharp'  : 'tqs',
        'sharp'                     : 's',
        'quarter sharp'           : 'qs',
        'natural'                   : '',
        'forced natural'          : '!',
        'quarter flat'            : 'qf',
        'flat'                        : 'f',
        'three-quarters flat'   : 'tqf',
        'double flat'             :  'ff',
    }

    _semitones_to_alphabetic_string = {
        -2   : 'ff',
        -1.5 : 'tqf',
        -1   : 'f',
        -0.5 : 'qf',
        0    : '',
        0.5  : 'qs',
        1    : 's',
        1.5  : 'tqs',
        2    : 'ss',
    }

    _symbolic_string_to_alphabetic_string = {
        ''   : '',
        '!'  : '!',
        'bb' : 'ff',
        'b+' : 'tqf',
        'b'  : 'f',
        'b-' : 'qf',
        '###' : 'ss',
        '#+' : 'tqs',
        '#'  : 's',
        '#-' : 'qs',
    }

    @property
    def _all_accidental_names(self):
        return self._name_to_alphabetic_string.keys()

    @property
    def _all_accidental_semitone_values(self):
        return self._semitones_to_alphabetic_string.keys()

    @property
    def _all_accidental_symbolic_strings(self):
        return self._symbolic_string_to_alphabetic_string.keys()

    ### PUBLIC ATTRIBUTES ###

    @property
    def alphabetic_string(self):
        '''Read-only alphabetic string::

            abjad> accidental = pitchtools.Accidental('s')
            abjad> accidental.alphabetic_string
            's'

        Return string.
        '''
        return self._alphabetic_string

    @property
    def format(self):
        '''Read-only LilyPond input format of accidental::

            abjad> accidental = pitchtools.Accidental('s')
            abjad> accidental.format
            's'

        Return string.
        '''
        return self._alphabetic_string

    @property
    def is_adjusted(self):
        '''True for all accidentals equal to a nonzero number of semitones. False otherwise::

            abjad> accidental = pitchtools.Accidental('s')
            abjad> accidental.is_adjusted
            True

        Return boolean.
        '''
        return self._is_adjusted

    @property
    def name(self):
        '''Read-only name string of accidental::

            abjad> accidental = pitchtools.Accidental('s')
            abjad> accidental.name
            'sharp'

        Return string.
        '''
        return self._name

    @property
    def semitones(self):
        '''Read-only semitones of accidental::

            abjad> accidental = pitchtools.Accidental('s')
            abjad> accidental.semitones
            1

        Return number.
        '''
        return self._semitones

    @property
    def symbolic_string(self):
        '''Read-only symbolic string of accidental::

            abjad> accidental = pitchtools.Accidental('s')
            abjad> accidental.symbolic_string
            '#'

        Return string.
        '''
        return self._symbolic_string




from abjad.tools.intervaltreetools import *


def test_IntervalTree___init___01():
    '''IntervalTree can be initialized without arguments.'''
    tree = IntervalTree([])

def tree_IntervalTree___init___02():
    '''IntervalTree can be instantiated from a single BoundedInterval.'''
    a = BoundedInterval(0, 10)
    tree = IntervalTree(a)
    assert a in tree

def tree_IntervalTree___init___03():
    '''IntervalTree can be instantiated from a single IntervalTree.'''
    a = BoundedInterval(0, 10)
    t = IntervalTree(a)
    tree = IntervalTree(t)
    assert a in tree

def test_IntervalTree___init___02():
    '''IntervalTree can be initialized from a list of BoundedIntervals.'''
    a = BoundedInterval(0, 10, {'a': 1})
    b = BoundedInterval(5, 15, {'b': 2})
    c = BoundedInterval(10, 25, {'c': 3})
    tree = IntervalTree([a, b, c])
    assert all([block in tree for block in [a, b, c]])

def test_IntervalTree___init___03():
    '''IntervalTree can be initialized from a list of BoundedIntervals.'''
    a = BoundedInterval(0, 10, {'a': 1})
    b = BoundedInterval(5, 15, {'b': 2})
    c = BoundedInterval(10, 25, {'c': 3})
    tree = IntervalTree([a, b, c])

def test_IntervalTree___init___04():
    '''IntervalTree recursively flattens its input argument,
    allowing instantiation from any nested collection of
    BoundedIntervals and / or trees.'''
    a = BoundedInterval(0, 10)
    b = BoundedInterval(5, 15)
    t = IntervalTree([a, b])
    c = BoundedInterval(21, 23)
    d = BoundedInterval(2001, 2009)
    tree = IntervalTree([a, b, [c, d], [[t]]])
    assert [x.signature for x in tree] == \
        [(0, 10), (0, 10), (5, 15), (5, 15), (21, 23), (2001, 2009)]



from abjad.tools.intervaltreetools.IntervalTree import IntervalTree
from abjad.tools.intervaltreetools.all_are_intervals_or_trees_or_empty import all_are_intervals_or_trees_or_empty


def get_all_unique_bounds_in_intervals(intervals):
    '''Return all unique starting and ending boundaries in `intervals`.'''

    assert all_are_intervals_or_trees_or_empty(intervals)
    if isinstance(intervals, IntervalTree):
        tree = intervals
    else:
        tree = IntervalTree(intervals)

    values = []
    for interval in tree:
        if interval.low not in values:
            values.append(interval.low)
        if interval.high not in values:
            values.append(interval.high)
    return tuple(sorted(values))



from abjad.core import _Immutable
import numbers


class PitchRange(_Immutable):
    r""".. versionadded:: 2.0

    Abjad model of pitch range::

        abjad> pitchtools.PitchRange(-12, 36)
        PitchRange((NamedChromaticPitch('c'), 'inclusive'), (NamedChromaticPitch("c''''"), 'inclusive'))

    Init from pitch numbers, pitch instances or other pitch range objects.

    Pitch ranges implement all six Python rich comparators.

    Pitch ranges are immutable.
    """

    def __init__(self, *args):
        from abjad.tools import pitchtools
        if len(args) == 0:
            object.__setattr__(self, '_start', None)
            object.__setattr__(self, '_stop', None)
        elif len(args) == 1 and isinstance(args[0], type(self)):
            if args[0].start_pitch_is_included_in_range:
                indicator = 'inclusive'
            else:
                indicator = 'exclusive'
            start = (args[0].start_pitch, indicator)
            object.__setattr__(self, '_start', start)
            assert isinstance(args[0], type(self))
            if args[0].stop_pitch_is_included_in_range:
                indicator = 'inclusive'
            else:
                indicator = 'exclusive'
            stop = (args[0].stop_pitch, indicator)
            object.__setattr__(self, '_stop', stop)
        elif len(args) == 1 and isinstance(args[0], (tuple, list)):
            start, stop = args[0]
            type(self).__init__(self, start, stop)
        else:
            assert len(args) == 2
            start, stop = args
            if start is None:
                start = start
            elif isinstance(start, (int, long, float)):
                pitch = pitchtools.NamedChromaticPitch(start)
                start = (pitch, 'inclusive')
            else:
                assert len(start) == 2
                pitch, containment = start
                assert containment in ('inclusive', 'exclusive')
                pitch = pitchtools.NamedChromaticPitch(pitch)
                start = (pitch, containment)
            object.__setattr__(self, '_start', start)
            if stop is None:
                stop = stop
            elif isinstance(stop, (int, long, float)):
                pitch = pitchtools.NamedChromaticPitch(stop)
                stop = (pitch, 'inclusive')
            else:
                assert len(stop) == 2
                pitch, containment = stop
                assert containment in ('inclusive', 'exclusive')
                pitch = pitchtools.NamedChromaticPitch(pitch)
                stop = (pitch, containment)
            object.__setattr__(self, '_stop', stop)

    ### OVERLOADS ###

    def __contains__(self, arg):
        from abjad.tools import containertools
        from abjad.tools import pitchtools
        from abjad.tools import resttools
        from abjad.tools import skiptools
        from abjad.tools.chordtools.Chord import Chord
        from abjad.tools.notetools.Note import Note
        if getattr(arg, 'written_pitch_indication_is_nonsemantic', False):
            return True
        elif isinstance(arg, (int, long, float)):
            pitch = pitchtools.NamedChromaticPitch(arg)
            return self._contains_pitch(pitch)
        elif isinstance(arg, pitchtools.NamedChromaticPitch):
            return self._contains_pitch(arg)
        elif isinstance(arg, Note):
            return self._contains_pitch(arg.sounding_pitch)
        elif isinstance(arg, Chord):
            return all([self._contains_pitch(x) for x in arg.sounding_pitches])
        elif isinstance(arg, (resttools.Rest, skiptools.Skip)):
            return True
        elif isinstance(arg, containertools.Container):
            return all([x in self for x in arg.leaves])
        else:
            pitches = pitchtools.list_named_chromatic_pitches_in_expr(arg)
            if pitches:
                return all([self._contains_pitch(x) for x in pitches])
            else:
                try:
                    return all([self._contains_pitch(x) for x in arg])
                except TypeError:
                    return False
        return False

    def __eq__(self, arg):
        if isinstance(arg, type(self)):
            if self._start == arg._start:
                if self._stop == arg._stop:
                    return True
        return False

    def __ge__(self, arg):
        from abjad.tools import pitchtools
        try:
            pitch = pitchtools.NamedChromaticPitch(arg)
            if self.start_pitch is None:
                return False
            return pitch <= self.start_pitch
        except (TypeError, ValueError):
            return False

    def __gt__(self, arg):
        from abjad.tools import pitchtools
        try:
            pitch = pitchtools.NamedChromaticPitch(arg)
            if self.start_pitch is None:
                return False
            return pitch < self.start_pitch
        except (TypeError, ValueError):
            return False

    def __le__(self, arg):
        from abjad.tools import pitchtools
        try:
            pitch = pitchtools.NamedChromaticPitch(arg)
            if self.stop_pitch is None:
                return False
            return self.stop_pitch <= pitch
        except (TypeError, ValueError):
            return False

    def __lt__(self, arg):
        from abjad.tools import pitchtools
        try:
            pitch = pitchtools.NamedChromaticPitch(arg)
            if self.stop_pitch is None:
                return False
            return self.stop_pitch < pitch
        except (TypeError, ValueError):
            return False

    def __ne__(self, arg):
        return not self == arg

    def __repr__(self):
        return '%s(%s, %s)' % (type(self).__name__, self._start, self._stop)

    ### PRIVATE METHODS ###

    def _contains_pitch(self, pitch):
        from abjad.tools import pitchtools
        if isinstance(pitch, numbers.Number):
            #pitch = pitchtools.NumberedChromaticPitch(pitch)
            pitch = pitchtools.NamedChromaticPitch(pitch)
        elif isinstance(pitch, str):
            pitch = pitchtools.NamedChromaticPitch(pitch)
        if self._start is None and self._stop is None:
            return True
        elif self._start is None:
            if self.stop_pitch_is_included_in_range:
                return pitch <= self.stop_pitch
            else:
                return pitch < self.stop_pitch
        elif self._stop is None:
            if self.start_pitch_is_included_in_range:
                return self.start_pitch <= pitch
            else:
                return self.start_pitch < pitch
        else:
            if self.start_pitch_is_included_in_range:
                if self.stop_pitch_is_included_in_range:
                    return self.start_pitch <= pitch <= self.stop_pitch
                else:
                    return self.start_pitch <= pitch < self.stop_pitch
            else:
                if self.stop_pitch_is_included_in_range:
                    return self.start_pitch < pitch <= self.stop_pitch
                else:
                    return self.start_pitch < pitch < self.stop_pitch

    ### PUBLIC ATTRIBUTES ###

    @property
    def start_pitch(self):
        r'''Read-only start pitch of range::

            abjad> pitch_range = pitchtools.PitchRange(-12, 36)
            abjad> pitch_range.start_pitch
            NamedChromaticPitch('c')

        Return pitch.
        '''
        if self._start is None:
            return None
        return self._start[0]

    @property
    def start_pitch_is_included_in_range(self):
        '''True when start pitch is included in range. Otherwise false::

            abjad> pitch_range = pitchtools.PitchRange(-12, 36)
            abjad> pitch_range.start_pitch_is_included_in_range
            True

        Return boolean.
        '''
        if self._start is None:
            return True
        return self._start[1] == 'inclusive'

    @property
    def stop_pitch(self):
        r"""Read-only stop pitch of range::

            abjad> pitch_range = pitchtools.PitchRange(-12, 36)
            abjad> pitch_range.stop_pitch
            NamedChromaticPitch("c''''")

        Return pitch.
        """
        if self._stop is None:
            return None
        return self._stop[0]

    @property
    def stop_pitch_is_included_in_range(self):
        '''True when stop pitch is included in range. Otherwise false::

            abjad> pitch_range = pitchtools.PitchRange(-12, 36)
            abjad> pitch_range.stop_pitch_is_included_in_range
            True

        Return boolean.
        '''
        if self._stop is None:
            return True
        return self._stop[1] == 'inclusive'




from abjad.core import _Immutable
from abjad.core import _StrictComparator


class SchemeColor(_StrictComparator, _Immutable):
    '''Abjad model of Scheme color::

        abjad> schemetools.SchemeColor('ForestGreen')
        SchemeColor('ForestGreen')

    Scheme colors are immutable.
    '''

    __slots__ = ('color_name',)

    def __new__(klass, color_name):
        self = object.__new__(klass)
        object.__setattr__(self, 'color_name', color_name)
        return self

    def __getnewargs__(self):
        return (self.color_name,)

    ### OVERLOADS ###

    def __repr__(self):
        return '%s(%r)' % (type(self).__name__, self.color_name)

    ### PUBLIC ATTRIBUTES ###

    @property
    def format(self):
        '''LilyPond input format of Scheme color::

            abjad> scheme_color = schemetools.SchemeColor('ForestGreen')
            abjad> scheme_color.format
            "#(x11-color 'ForestGreen)"

        Return string.
        '''
        return "#(x11-color '%s)" % self.color_name



def _write_file_interactive(content, file, interactive):
   if interactive.lower in ('y', ''):
      msg = 'Will write file "%s". Proceed? [Y/n]:' % file
      input = raw_input(msg)
   else:
      input = 'Y'
   if input.lower() in ('y', ''):
      #print 'Writing file %s ...' % file
      # if file already exists,
      # do not overwrite unless content has changed.
      try:
         prev_file = open(file, 'r')
         prev_content = prev_file.read()
         prev_file.close()
         if content != prev_content:
            listing_file = open(file, 'w')
            listing_file.write(content)
            listing_file.close()
      except:
         listing_file = open(file, 'w')
         listing_file.write(content)
         listing_file.close()



from abjad.core._StrictComparator import _StrictComparator


class _FlexEqualityComparator(_StrictComparator):
    '''Mix-in base class to confer a certain type of comparator behavior to any custom class.
    Flex equality comparators raise not implemented error on gt, lt, ge, le.
    Flex equality comparators attempt type coercion on eq, ne.
    '''

    __slots__ = ('_comparison_attribute', '_format_string')

    ### OVERLOADS ###

    def __eq__(self, arg):
        arg = self._massage_comparison_arg(arg)
        return arg and self._comparison_attribute == arg._comparison_attribute

    def __ne__(self, arg):
        return not self == arg

    def __repr__(self):
        return '%s(%s)' % (self.__class__.__name__, self._format_string)

    ### PRIVATE METHODS ###

    def _massage_comparison_arg(self, arg):
        if not isinstance(arg, type(self)):
            try:
                arg = type(self)(arg)
            except (ValueError, TypeError):
                return False
        return arg




from abjad.tools.lilyfiletools._BlockAttributed import _BlockAttributed


class MidiBlock(_BlockAttributed):
    r'''.. versionadded:: 2.0

    Abjad model of LilyPond input file midi block.
    '''

    def __init__(self):
        _BlockAttributed.__init__(self)
        self._escaped_name = r'\midi'
        self.is_formatted_when_empty = True



from abjad.tools import pitchtools
from abjad.tools.instrumenttools.Flute import Flute


class ContrabassFlute(Flute):
    r'''.. versionadded:: 2.0

    Abjad model of the contrabass flute::

        abjad> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        abjad> instrumenttools.ContrabassFlute()(staff)
        ContrabassFlute('Contrabass Flute', 'Cbass Fl.')(Staff{4})

    ::

        abjad> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { Contrabass Flute }
            \set Staff.shortInstrumentName = \markup { Cbass Fl. }
            c'8
            d'8
            e'8
            f'8
        }

    The contrabass flute targets staff context by default.
    '''

    def __init__(self,
        instrument_name = r'Contrabass Flute',
        short_instrument_name = r'Cbass Fl.',
        target_context = None):
        Flute.__init__(self, instrument_name, short_instrument_name, target_context)
        self.sounding_pitch_of_fingered_middle_c = pitchtools.NamedChromaticPitch('g,')
        self.traditional_range = (-17, 19)



from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals
from abjad import Fraction
import py.test


def test_intervaltreetools_scale_interval_magnitudes_by_rational_01():
    a = BoundedInterval(0, 10, {'a': 1})
    b = BoundedInterval(Fraction(5, 3), 10, {'b': 2})
    c = BoundedInterval(5, 10, {'c': 3})
    tree = IntervalTree([a, b, c])
    scalar = Fraction(5, 2)
    scaled = scale_interval_magnitudes_by_rational(tree, scalar)
    assert scaled[0] == BoundedInterval(0, Fraction(25, 1), {'a': 1})
    assert scaled[1] == BoundedInterval(Fraction(5, 3), Fraction(45, 2), {'b': 2})
    assert scaled[2] == BoundedInterval(5, Fraction(35, 2), {'c': 3})
    assert scaled.magnitude == (scalar * tree.magnitude)


def test_intervaltreetools_scale_interval_magnitudes_by_rational_02():
    tree = IntervalTree([])
    scalar = Fraction(5, 2)
    scaled = scale_interval_magnitudes_by_rational(tree, scalar)
    assert scaled == tree



def _get_markup_format_contributions(component):
    '''.. versionadded:: 2.0
    '''
    from abjad.tools import markuptools

    result = []
    markup = markuptools.get_markup_attached_to_component(component)
    up_markup, down_markup, neutral_markup = [], [], []
    for markup_object in markup:
        if markup_object._direction_string == 'up':
            up_markup.append(markup_object)
        elif markup_object._direction_string == 'down':
            down_markup.append(markup_object)
        elif markup_object._direction_string in ('neutral', None):
            neutral_markup.append(markup_object)
    for markup_list in (up_markup, down_markup, neutral_markup):
        if not markup_list:
            pass
        elif 1 < len(markup_list):
            #markup_list.sort()
            #contents = ' '.join([m._contents_string for m in markup_list])
            contents = [m._contents_string for m in markup_list]
#         contents.sort()
            contents = ' '.join(contents)
            direction_symbol = m._direction_string_to_direction_symbol[m._direction_string]
            column = r'%s \markup { \column { %s } }' % (direction_symbol, contents)
            result.append(column)
        else:
            result.append(markup_list[0].format)
    return ['markup', result]



class _Immutable(object):
    '''.. versionadded:: 2.0

    Base from which immutable custom classes can inherit.

    .. todo:: write code to check all Abjad system objects with slots and
        make sure none has a dict defined and taking up memory in the namespace of the object.
    '''

    __slots__ = ( )

    ### OVERLOADS ###

    def __copy__(self, *args):
        return type(self)(self)

    __deepcopy__ = __copy__

    def __delattr__(self, *args):
        raise AttributeError('objects are immutable: "%s".' % self.__class__.__name__)

    def __getstate__(self):
        return { }

    def __setattr__(self, *args):
        raise AttributeError('objects are immutable: "%s".' % self.__class__.__name__)

    def __setstate__(self, state):
        pass






class CyclicList(list):
    '''.. versionadded:: 2.0

    Abjad model of cyclic list::

        abjad> from abjad.tools import sequencetools

    ::

        abjad> cyclic_list = sequencetools.CyclicList('abcd')

    ::

        abjad> cyclic_list
        ['a', 'b', 'c', 'd']

    ::

        abjad> for x in range(8):
        ...     print x, cyclic_list[x]
        ...
        0 a
        1 b
        2 c
        3 d
        4 a
        5 b
        6 c
        7 d

    Cyclic lists overload the item-getting method of built-in lists.

    Cyclic lists return a value for any integer index.

    Cyclic lists otherwise behave exactly like built-in lists.
    '''

    ### OVERLOADS ###

    def __getitem__(self, expr):
        return list.__getitem__(self, expr % len(self))

    def __getslice__(self, start_index, stop_index):
        result = []
        result = [self[n] for n in range(start_index, stop_index)]
        return result



from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.containertools.Container import Container
from abjad.tools.tuplettools.Tuplet._TupletFormatter import _TupletFormatter
import fractions


class Tuplet(Container):
    r'''Abjad model of a tuplet:

    ::

        abjad> tuplet = Tuplet(Fraction(2, 3), "c'8 d'8 e'8")
        abjad> f(tuplet)
        \times 2/3 {
            c'8
            d'8
            e'8
        }

    Return tuplet object.
    '''

    __slots__ = ('_force_fraction', '_is_invisible', '_multiplier', '_preferred_denominator',
        '_signifier', )

    def __init__(self, multiplier, music = None, **kwargs):
        Container.__init__(self, music)
        self.multiplier = multiplier
        self._force_fraction = None
        self._formatter = _TupletFormatter(self)
        self._is_invisible = None
        self._preferred_denominator = None
        self._signifier = '*'
        self._initialize_keyword_values(**kwargs)

    ### OVERLOADS ###

    def __add__(self, arg):
        '''Add two tuplets of same type and with same multiplier.
        '''
        from abjad.tools import tuplettools
        assert isinstance(arg, type(self))
        new = tuplettools.fuse_tuplets([self, arg])
        return new

    def __getnewargs__(self):
        return (self.multiplier, )

    def __repr__(self):
        return '%s(%s, [%s])' % (type(self).__name__, self.multiplier, self._summary)

    def __str__(self):
        if 0 < len(self):
            return '{%s %s %s %s}' % (self._signifier, self.ratio, self._summary, self._signifier)
        else:
            return '{%s %s %s}' % (self._signifier, self.multiplier, self._signifier)

    ### PRIVATE ATTRIBUTES ###

    @property
    def _is_visible(self):
        return not self.is_invisible

    @property
    def _multiplier_fraction_string(self):
        if self.preferred_denominator is not None:
            inverse_multiplier = fractions.Fraction(
                self.multiplier.denominator, self.multiplier.numerator)
            d, n = durationtools.rational_to_duration_pair_with_specified_integer_denominator(
                inverse_multiplier, self.preferred_denominator)
        else:
            n, d = self.multiplier.numerator, self.multiplier.denominator
        return '%s/%s' % (n, d)

    @property
    def _summary(self):
        if 0 < len(self):
            return ', '.join([str(x) for x in self._music])
        else:
            #return ' '
            return ''

    ### PUBLIC ATTRIBUTES ###

    @apply
    def force_fraction():
        def fget(self):
            '''Read / write boolean to force ``n:m`` fraction.
            '''
            return self._force_fraction
        def fset(self, arg):
            if isinstance(arg, (bool, type(None))):
                self._force_fraction = arg
            else:
                raise TypeError('bad type for tuplet force fraction: "%s".' % arg)
        return property(**locals())

    @property
    def is_augmentation(self):
        '''True when multiplier is greater than 1.
        Otherwise false::

            abjad> t = tuplettools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
            abjad> t.is_augmentation
            False

        Return boolean.
        '''
        if self.multiplier:
            return 1 < self.multiplier
        else:
            return False

    @property
    def is_binary(self):
        '''True when multiplier numerator is power of two, otherwise False.
        '''
        if self.multiplier:
            return mathtools.is_nonnegative_integer_power_of_two(self.multiplier.numerator)
        else:
            return True

    @property
    def is_diminution(self):
        '''True when multiplier is less than 1.  Otherwise false::

            abjad> t = tuplettools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
            abjad> t.is_diminution
            True

        Return boolean.
        '''
        if self.multiplier:
            return self.multiplier < 1
        else:
            return False

    @apply
    def is_invisible():
        def fget(self):
            '''Read / write boolean to render tuplet invisible.
            '''
            return self._is_invisible
        def fset(self, arg):
            assert isinstance(arg, (bool, type(None)))
            self._is_invisible = arg
        return property(**locals())

    @property
    def is_nonbinary(self):
        return not self.is_binary

    @property
    def is_trivial(self):
        '''True when tuplet multiplier is one, otherwise False.
        '''
        return self.multiplier == 1

    @property
    def multiplied_duration(self):
        return self.multiplier * self.contents_duration

    @apply
    def multiplier():
        def fget(self):
            return self._multiplier
        def fset(self, expr):
            if isinstance(expr, (int, long)):
                rational = fractions.Fraction(expr)
            elif isinstance(expr, tuple):
                rational = fractions.Fraction(*expr)
            elif isinstance(expr, fractions.Fraction):
                rational = fractions.Fraction(expr)
            else:
                raise ValueError('can not set tuplet multiplier: "%s".' % str(expr))
            if 0 < rational:
                self._multiplier = rational
            else:
                raise ValueError('tuplet multiplier must be positive: "%s".' % rational)
        return property(**locals())

    @apply
    def preferred_denominator():
        def fget(self):
            '''.. versionadded:: 2.0

            Integer denominator in terms of which tuplet fraction should format.
            '''
            return self._preferred_denominator
        def fset(self, arg):
            if isinstance(arg, (int, long)):
                if not 0 < arg:
                    raise ValueError('tuplet preferred denominator must be positive: "%s".' % arg)
            elif not isinstance(arg, type(None)):
                raise TypeError('bad tuplet preferred denominator type: "%s".' % arg)
            self._preferred_denominator = arg
        return property(**locals())

    @property
    def preprolated_duration(self):
        '''Duration prior to prolation:

        ::

            abjad> t = tuplettools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
            abjad> t.preprolated_duration
            Duration(1, 4)

        Return duration.
        '''
        return self.multiplied_duration

    @property
    def ratio(self):
        '''Tuplet multiplier formatted with colon as ratio.
        '''
        multiplier = self.multiplier
        if multiplier is not None:
            return '%s:%s' % (multiplier.denominator, multiplier.numerator)
        else:
            return None



from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import pitchtools
from abjad.tools import sequencetools
from abjad.tools import durationtools
from numbers import Number
import fractions
import math
import operator


def make_notes(pitches, durations, direction='big-endian'):
    '''Make notes according to `pitches` and `durations`.

    Cycle through `pitches` when the length of `pitches` is less than the length of `durations`::

        abjad> notetools.make_notes([0], [(1, 16), (1, 8), (1, 8)])
        [Note("c'16"), Note("c'8"), Note("c'8")]

    Cycle through `durations` when the length of `durations` is less than the length of `pitches`::

        abjad> notetools.make_notes([0, 2, 4, 5, 7], [(1, 16), (1, 8), (1, 8)])
        [Note("c'16"), Note("d'8"), Note("e'8"), Note("f'16"), Note("g'8")]

    Create ad hoc tuplets for nonassignable durations::

        abjad> notetools.make_notes([0], [(1, 16), (1, 12), (1, 8)])
        [Note("c'16"), Tuplet(2/3, [c'8]), Note("c'8")]

    Set `direction` to ``'big-endian'`` to express tied values in decreasing duration::

        abjad> notetools.make_notes([0], [(13, 16)], direction = 'big-endian')
        [Note("c'2."), Note("c'16")]

    Set `direction` to ``'little-endian'`` to express tied values in increasing duration::

        abjad> notetools.make_notes([0], [(13, 16)], direction = 'little-endian')
        [Note("c'16"), Note("c'2.")]

    Set `pitches` to a single pitch or a sequence of pitches.

    Set `durations` to a single duration or a list of durations.

    Return list of newly constructed notes.

    .. versionchanged:: 2.0
        renamed ``construct.notes()`` to
        ``notetools.make_notes()``.
    '''
    from abjad.tools.leaftools._construct_unprolated_notes import _construct_unprolated_notes
    from abjad.tools.tuplettools.Tuplet import Tuplet

    if pitchtools.is_named_chromatic_pitch_token(pitches):
        pitches = [pitches]

    #if durationtools.is_duration_token(durations):
    if isinstance(durations, (Number, tuple)):
        durations = [durations]

    # this block is a hack to allow the function to accept a Duration
    # as the duration input parameter; better will be to change
    # the rest of the implementation to allow for Durations directly.
    # [VA] We don't want to change to Durations internally because
    # Durations reduce fractions to their minimum expression. e.g.
    # (3, 3) --> Duration(1, 1), and we sometimes generate duration
    # tokens that are not reduced, so we want to preserve the denominator 3.
    # [TB] When do we want (3, 3) instead of (1, 1)?
    # Durations should always reduce;
    # So tokens can represent tuplet multipliers or something
    # else that shouldn't reduce?
    durations = [durationtools.duration_token_to_duration_pair(dur) for dur in durations]

    # set lists of pitches and durations to the same length
    size = max(len(durations), len(pitches))
    durations = sequencetools.repeat_sequence_to_length(durations, size)
    pitches = sequencetools.repeat_sequence_to_length(pitches, size)

    durations = durationtools.group_duration_tokens_by_implied_prolation(durations)

    result = []
    for ds in durations:
        # get factors in denominator of duration group ds other than 1, 2.
        factors = set(mathtools.factors(ds[0][1]))
        factors.discard(1)
        factors.discard(2)
        ps = pitches[0:len(ds)]
        pitches = pitches[len(ds):]
        if len(factors) == 0:
            result.extend(_construct_unprolated_notes(ps, ds, direction))
        else:
            # compute prolation
            #denominator = reduce(operator.mul, factors)
            denominator = ds[0][1]
            numerator = mathtools.greatest_power_of_two_less_equal(denominator)
            multiplier = (numerator, denominator)
            ratio = 1 / fractions.Fraction(*multiplier)
            ds = [ratio * durationtools.Duration(*d) for d in ds]
            # make notes
            ns = _construct_unprolated_notes(ps, ds, direction)
            #t = Tuplet(multiplier, ns)
            t = Tuplet(multiplier, ns)
            result.append(t)
    return result





from abjad.tools.intervaltreetools.BoundedInterval import BoundedInterval
from abjad.tools.intervaltreetools.IntervalTree import IntervalTree
from abjad.tools.intervaltreetools.all_are_intervals_or_trees_or_empty import all_are_intervals_or_trees_or_empty
from abjad.tools.intervaltreetools.group_tangent_or_overlapping_intervals_and_yield_groups import group_tangent_or_overlapping_intervals_and_yield_groups

def fuse_tangent_or_overlapping_intervals(intervals):
    '''Fuse all tangent or overlapping intervals and return an `IntervalTree` of the result ::

        abjad> from abjad.tools import intervaltreetools
        abjad> from abjad.tools.intervaltreetools import BoundedInterval
        abjad> from abjad.tools.intervaltreetools import IntervalTree

    ::

        abjad> a = BoundedInterval(0, 10)
        abjad> b = BoundedInterval(5, 15)
        abjad> c = BoundedInterval(15, 25)
        abjad> tree = IntervalTree([a, b, c])
        abjad> intervaltreetools.fuse_tangent_or_overlapping_intervals(tree)
        IntervalTree([
            BoundedInterval(Offset(0, 1), Offset(25, 1), {})
        ])

    Return interval tree.
    '''

    assert all_are_intervals_or_trees_or_empty(intervals)
    if isinstance(intervals, IntervalTree):
        tree = intervals
    else:
        tree = IntervalTree(intervals)
    if not tree:
        return tree

    trees = [IntervalTree(group) for group in \
        group_tangent_or_overlapping_intervals_and_yield_groups(tree)]

    return IntervalTree([
        BoundedInterval(tree.low_min, tree.high_max) for tree in trees
    ])



from abjad.core import _Immutable


class SchemePair(tuple, _Immutable):
    '''Abjad model of Scheme pair::

        abjad> schemetools.SchemePair('spacing', 4)
        SchemePair('spacing', 4)

    Initialize Scheme pairs with a tuple, two separate values or another Scheme pair.

    Scheme pairs are immutable.
    '''

    def __new__(klass, *args):
        if len(args) == 1 and isinstance(args[0], SchemePair):
            self = tuple.__new__(klass, args[0][:])
        elif len(args) == 1 and isinstance(args[0], tuple):
            self = tuple.__new__(klass, args[0][:])
        elif len(args) == 2:
            self = tuple.__new__(klass, args)
        else:
            raise TypeError('can not initialize Scheme pair from "%s".' % str(args))
        return self

    def __getnewargs__(self):
        return tuple(self)

    ### OVERLOADS ###

    def __repr__(self):
        return '%s(%s)' % (type(self).__name__, self._format_string)

    def __str__(self):
        return '(%s)' % self._output_string

    ### PRIVATE ATTRIBUTES ###

    @property
    def _format_string(self):
        #return ', '.join([str(x) for x in self])
        result = []
        for x in self:
            if isinstance(x, str):
                result.append('%r' % x)
            else:
                result.append(str(x))
        result = ', '.join(result)
        return result

    @property
    def _output_string(self):
        vals = []
        for x in self:
            if isinstance(x, type(True)) and x:
                vals.append("#t")
            elif isinstance(x, type(True)):
                vals.append("#f")
            else:
                vals.append(x)
        return '%s . %s' % (vals[0], vals[1])

    ### PUBLIC ATTRIBUTES ###

    @property
    def format(self):
        '''LilyPond input format of Scheme pair::

            abjad> scheme_pair = schemetools.SchemePair('spacing', 4)
            abjad> scheme_pair.format
            "#'(spacing . 4)"

        Return string.
        '''
        return "#'%s" % self.__str__()



from abjad import *


def test_SchemeBoolean___init___01():

    t = Note(0, (1, 16))
    t.override.stem.transparent = schemetools.SchemeBoolean(True)

    r'''
    \once \override Stem #'transparent = ##t
    c'16
    '''

    assert t.format == '\\once \\override Stem #\'transparent = ##t\nc\'16'



import copy
from collections import Iterable
from numbers import Number
from abjad import Container
from abjad import Fraction
from abjad import Note
from abjad import Tuplet
from abjad.core import _Immutable
from abjad.tools.durationtools import Offset
from abjad.tools.mathtools import divisors
from abjad.tools.mathtools import greatest_power_of_two_less_equal
from abjad.tools.quantizationtools.QEvent import QEvent
from abjad.tools.quantizationtools.is_valid_beatspan import is_valid_beatspan
from abjad.tools.sequencetools import all_are_numbers
from abjad.tools.sequencetools import flatten_sequence


class QGrid(_Immutable):
    '''Abjad model of a QGrid, a nesting division structure which
    assists certain quantization algorithms.

    `QGrids` are defined by a list, which must be prime in length,
    whose members are either Numbers or tuples of Numbers (useful for
    representing timepoint or pitch information), a
    :py:class:`~abjad.tools.quantizationtools.QEvent` or tuple of
    :py:class:`~abjad.tools.quantizationtools.QEvent` objects, or None
    (representing silence), or other lists which must recursively
    obey the same rules.

    `QGrids` also have a `next` attribute, representing the downbeat of
    not "this" `QGrid`, but the next `QGrid` in a list of grids.  This is
    useful as timepoints must often be quantized not to any internal
    division of a the "current" beat, but to the next beat.

    ::

        abjad> from abjad.tools.quantizationtools import QGrid
        abjad> q = QGrid([0, 0, [0, 0]], 0)

    The values in the grid can be access via subscript, as though
    the grid were a flat list.

    ::

        abjad> q[0] = 1
        abjad> q[2] = 3
        abjad> q[4] = 5
        abjad> q
        QGrid([1, 0, [3, 0]], 5)

    `QGrids` are quasi-immutable.
    '''

    __slots__ = ('_definition', '_next', '_offsets')

    def __new__(klass, definition, next):
        self = object.__new__(klass)
        assert self._is_valid_grid_definition(definition)
        assert self._is_valid_grid_value(next)
        object.__setattr__(self, '_definition', definition)
        object.__setattr__(self, '_next', next)
        object.__setattr__(self, '_offsets', self._expand_offsets())
        return self

    def __getnewargs__(self):
        return self._definition, self._next

    # OVERRIDES #

    def __eq__(self, other):
        if type(self) == type(other) and \
            self._definition == other._definition and \
            self._next == other._next:
            return True
        return False

    def __getitem__(self, item):
        if not isinstance(item, int):
            return None
        if item < 0:
            item = len(self) + item
        if item < 0 or len(self) <= item:
            return None

        for i, x in enumerate(self):
            if i == item:
                return x

    def __iter__(self):
        seq = flatten_sequence(self._definition, klasses=list)
        seq.append(self._next)
        for x in seq:
            yield x

    def __len__(self):
        return len(self.offsets)

    def __repr__(self):
        return '%s(%s)' % (type(self).__name__, self._format_string)

    def __setitem__(self, item, value):
        def recurse(n, item, value, prev_count):
            count = prev_count
            for i, x in enumerate(n):
                if isinstance(x, list):
                    count = recurse(x, item, value, count)
                else:
                    if count == item:
                        n[i] = value
                    count += 1
            return count

        if not isinstance(item, int):
            raise ValueError('Index must be an int.')
        if item < 0:
            item = len(self) + item
        if item < 0 or len(self) <= item:
            raise Exception('Index out of bounds.')

        if not self._is_valid_grid_value(value):
            raise ValueError

        if item == len(self) - 1:
            object.__setattr__(self, '_next', value)
        else:
            recurse(self._definition, item, value, 0)

    ### PRIVATE ATTRIBUTES ###

    @property
    def _format_string(self):
        return '%s, %s' % (self._definition, self._next)

    ### PRIVATE METHODS ###

    def _expand_offsets(self):
        def recurse(n, prev_div, prev_offset):
            results = []
            this_div = Fraction(1, len(n)) * prev_div
            for i, x in enumerate(n):
                this_offset = Offset(i * this_div) + prev_offset
                if isinstance(x, list):
                    results.extend(recurse(x, this_div, this_offset))
                else:
                    results.append(this_offset)
            return results
        expanded = list(recurse(self._definition, 1, 0))
        expanded.append(Offset(1))
        return tuple(expanded)

    def _is_valid_grid_definition(self, definition):
        def recurse(n):
            if not isinstance(n, list) or \
                not all([isinstance(x, list) or \
                    self._is_valid_grid_value(x) for x in n]) or \
                not set(divisors(len(n))) == set([1, len(n)]):
                return [False]
            results = []
            for x in n:
                if isinstance(x, list):
                    results.extend(recurse(x))
                else:
                    results.append(self._is_valid_grid_value(x))
            return results
        return all(recurse(definition))

    def _is_valid_grid_value(self, value):
        if isinstance(value, (Number, type(None), QEvent)):
            return True
        elif isinstance(value, tuple):
            if 0 < len(value):
                if all([isinstance(x, Number) for x in value]) or \
                    all([isinstance(x, QEvent) for x in value]):
                    return True
            else:
                return True
        return False

    ### PUBLIC ATTRIBUTES ###

    @property
    def definition(self):
        '''The nested list which defines the `QGrid's` structure.

        ::

            abjad> from abjad.tools.quantizationtools import QGrid
            abjad> q = QGrid([0, 0, [0, 0]], 0)
            abjad> q.definition
            [0, 0, [0, 0]]

        Read-only.
        '''

        return copy.deepcopy(self._definition)

    @property
    def next(self):
        '''The contents of the final offset in the `QGrid.`

        ::

            abjad> from abjad.tools.quantizationtools import QGrid
            abjad> q = QGrid([0, 0, [0, 0]], 0)
            abjad> q[-1] = 9
            abjad> q
            QGrid([0, 0, [0, 0]], 9)
            abjad> q.next
            9

        Read-only.
        '''

        return self._next

    @property
    def offsets(self):
        '''An ordered tuple of those :py:class:`~abjad.tools.durationtools.Offset`
        objects generated by the division structure of a `QGrid`.

        ::

            abjad> from abjad.tools.quantizationtools import QGrid
            abjad> q = QGrid([0, [0, 0], 0], 0)
            abjad> q.offsets
            (Offset(0, 1), Offset(1, 3), Offset(1, 2), Offset(2, 3), Offset(1, 1))

        Read-only.
        '''

        return self._offsets

    ### PUBLIC METHODS ###

    def find_parentage_of_index(self, index):
        '''Return a tuple of the lengths of each container containing `index`,
        from the topmost to the bottommost.

        ::

            abjad> from abjad.tools.quantizationtools import QGrid
            abjad> q = QGrid([0, [0, [0, 0], 0], 0, 0, 0], 0)
            abjad> q.find_parentage_of_index(0)
            (5,)
            abjad> q.find_parentage_of_index(1)
            (5, 3)
            abjad> q.find_parentage_of_index(2)
            (5, 3, 2)
            abjad> q.find_parentage_of_index(7)
            (5,)

        Returns a tuple.
        '''

        if index < 0:
            index = len(self) + index
        if index < 0 or len(self) <= index:
            return None
        if index == len(self) - 1:
            return None
        def recurse(n, index, prev_count):
            results = []
            count = prev_count
            for i, x in enumerate(n):
                if isinstance(x, list):
                    subcount, subresults = recurse(x, index, count)
                    count = subcount
                    if subresults:
                        results = [len(n)]
                        results.extend(subresults)
                        return count, results
                else:
                    if index == count:
                        return count, [len(n)]
                    count += 1
            return count, results
        return tuple(recurse(self.definition, index, 0)[1])

    def find_divisible_indices(self, points):
        '''Given a list of numbers 0 <= n <= 1, return a list of indices in self
        which countain those points, as though they were segments.

        ::

            abjad> from abjad.tools.quantizationtools import QGrid
            abjad> q = QGrid([0, [0, 0]], 0)
            abjad> q.offsets
            (Offset(0, 1), Offset(1, 2), Offset(3, 4), Offset(1, 1))
            abjad> points = [0.1, 0.9]
            abjad> q.find_divisible_indices(points)
            [0, 2]

        Returns a list.
        '''

        assert all_are_numbers(points)
        points = filter(lambda x: 0 <= x <= 1, points)
        offsets = self.offsets
        indices = []
        for i in range(len(offsets) - 1):
            filtered = filter(lambda x: offsets[i] < x < offsets[i + 1], points)
            if filtered:
                indices.append(i)
        return indices

    def format_for_beatspan(self, beatspan = Fraction(1, 4)):
        '''Return an Abjad container, whose structure mirrors the
        division structure of the `QGrid`.  The values of the items in
        the `QGrid` have no effect on the output.

        ::

            abjad> from abjad.tools.quantizationtools import QGrid
            abjad> q = QGrid([0, [0, 0], 0], 0)
            abjad> q.format_for_beatspan()
            Tuplet(2/3, [c'8, c'16, c'16, c'8])

        Returns a :py:class:`~abjad.tools.tuplettools.Tuplet` or
        :py:class:`~abjad.tools.containertools.Container`, depending on
        structure.
        '''

        assert is_valid_beatspan(beatspan)

        def recurse(n, division):
            pow = greatest_power_of_two_less_equal(len(n))
            val = Fraction(1, pow) * division
            if divisors(len(n)) in [[1], [1, 2]]:
                c = Container([])
            else: # we are in a non-2 prime container, hence tuplet
                c = Tuplet(Fraction(pow, len(n)), [])
            for x in n:
                if isinstance(x, list):
                    if len(x) in [1, 2]:
                        c.extend(recurse(x, val))
                    else:
                        c.append(recurse(x, val))
                else:
                    c.append(Note(0, val))
            return c
        return recurse(self.definition, beatspan)

    def subdivide_indices(self, pairs):
        '''Given a list of 2-tuples, where for each tuple t,
        t[0] is a valid index into self, and t[1] is a prime integer
        greater than 1, return a new `QGrid` with those indices subdivided.

        ::

            abjad> from abjad.tools.quantizationtools import QGrid
            abjad> q = QGrid([0, 0], 0)
            abjad> q.subdivide_indices([(0, 2), (1, 3)])
            QGrid([[0, 0], [0, 0, 0]], 0)

        Returns a new `QGrid`.
        '''

        # add some validation here
        assert isinstance(pairs, Iterable) and \
            all([isinstance(x, Iterable) and len(x) == 2 for x in pairs]) and \
            all([0 <= x[0] < len(self) - 1 for x in pairs]) and \
            all([set(divisors(x[1])) == set([1, x[1]]) for x in pairs])
        pairs = sorted(pairs, key = lambda x: x[0])
        def recurse(n, prev_count):
            count = prev_count
            for i, x in enumerate(n):
                if isinstance(x, list):
                    count = recurse(x, count)
                else:
                    for pair in filter(lambda x: x[0] == count, pairs):
                        n[i] = [0] * pair[1]
                        pairs.pop(pairs.index(pair))
                    count += 1
            return count
        definition = copy.deepcopy(self.definition)
        recurse(definition, 0)
        return QGrid(definition, self.next)



from abjad.tools.intervaltreetools.BoundedInterval import BoundedInterval
from abjad.tools.intervaltreetools.IntervalTree import IntervalTree
from abjad.tools.intervaltreetools.all_are_intervals_or_trees_or_empty import all_are_intervals_or_trees_or_empty
from abjad import Fraction


def calculate_density_of_attacks_in_interval(intervals, interval):
    '''Return a Fraction of number of attacks in `interval`
    over the magnitude of `interval`.'''

    assert all_are_intervals_or_trees_or_empty(intervals)
    assert isinstance(interval, BoundedInterval)
    if isinstance(intervals, IntervalTree):
        tree = intervals
    else:
        tree = IntervalTree(intervals)

    return len(tree.find_intervals_starting_within_interval(interval)) \
        / interval.magnitude



class _ImmutableDictionary(dict):
    '''.. versionadded:: 2.0
    '''

    def __delitem__(self, *args):
        raise AttributeError('objects are immutable: "%s".' % self.__class__.__name__)

    def __setitem__(self, *args):
        raise AttributeError('objects are immutable: "%s".' % self.__class__.__name__)




from abjad import *


def test_SchemeVariable___init___01():

    t = Note("c'4")
    t.override.stem.direction = schemetools.SchemeVariable('DOWN')

    r'''
    \once \override Stem #'direction = #DOWN
    c'4
    '''

    assert t.format == "\\once \\override Stem #'direction = #DOWN\nc'4"



from abjad import *


def test_instrumenttools_transpose_notes_and_chords_in_expr_from_sounding_pitch_to_fingered_pitch_01():

    staff = Staff("<c' e' g'>4 d'4 r4 e'4")
    instrumenttools.Clarinet()(staff)

    for leaf in staff.leaves:
        if isinstance(leaf, (Note, Chord)):
            assert leaf.written_pitch_indication_is_at_sounding_pitch

    instrumenttools.transpose_notes_and_chords_in_expr_from_sounding_pitch_to_fingered_pitch(staff)

    r'''
    \new Staff {
        \set Staff.instrumentName = \markup { Clarinet }
        \set Staff.shortInstrumentName = \markup { Cl. }
        <d' fs' a'>4
        e'4
        r4
        fs'4
    }
    '''

    for leaf in staff.leaves:
        if isinstance(leaf, (Note, Chord)):
            assert not leaf.written_pitch_indication_is_at_sounding_pitch

    assert staff.format == "\\new Staff {\n\t\\set Staff.instrumentName = \\markup { Clarinet }\n\t\\set Staff.shortInstrumentName = \\markup { Cl. }\n\t<d' fs' a'>4\n\te'4\n\tr4\n\tfs'4\n}"



from abjad.tools.componenttools.all_are_components import all_are_components
from abjad.tools.componenttools.get_parent_and_start_stop_indices_of_components import get_parent_and_start_stop_indices_of_components


def extend_left_in_parent_of_component_and_do_not_grow_spanners(component, components):
    r'''.. versionadded:: 1.1

    Extend `components` left in parent of `component` and do not grow spanners::

        abjad> notes = [Note("c'8"), Note("d'8"), Note("e'8")]
        abjad> t = Voice(notes)
        abjad> spannertools.BeamSpanner(t[:])
        BeamSpanner(c'8, d'8, e'8)
        abjad> notes = [Note("c'8"), Note("d'8"), Note("e'8")]
        abjad> componenttools.extend_left_in_parent_of_component_and_do_not_grow_spanners(t[0], notes)
        [Note("c'8"), Note("d'8"), Note("e'8"), Note("c'8")]

    ::

        abjad> print t.format
        \new Voice {
            c'8
            d'8
            e'8
            c'8 [
            d'8
            e'8 ]
        }

    Return `components` and `component` together in newly created list.

    .. versionchanged:: 2.0 renamed ``extend_left_in_parent()`` to
        ``extend_left_in_parent_of_component_and_do_not_grow_spanners()``.
    '''

    assert all_are_components(components)
    parent, start, stop = get_parent_and_start_stop_indices_of_components([component])
    if parent is not None:
        parent[start:start] = components
    return components + [component]



import os


_fns = os.listdir(__path__[0])
_modules = [_fn[:-3] for _fn in _fns 
   if _fn.endswith('.py') and not _fn.startswith('_')]

for _module in sorted(_modules):
   exec('from %s import *' % _module)

def check_cleaner(locals):
   #from types import TypeType
   import types
   from abjad.checks._Check import _Check
   for _key, _value in locals.items( ):
      #if not isinstance(_value, types.TypeType) and _key != 'check_cleaner':
      #   del(locals[_key])
      if _key == 'check_cleaner' or \
         (isinstance(_value, types.TypeType) and \
         'checks' in _value.__module__):
         pass
      else:
         del(locals[_key])
check_cleaner(locals( ))
del check_cleaner



from _get_documenting_names import _get_documenting_names
from _module_path_to_doc_path import _module_path_to_doc_path
import os


def make_sphinx_toc():
   '''Make table of contents for Abjad API.
   Divide TOC into classes, interfaces, spanners and tools.
   Divide tools into classes and functions.
   '''

   print 'Now making Sphinx TOC ...'
   names = _get_documenting_names()
   klasses, interfaces, spanners, exceptions, tools = [ ], [ ], [ ], [ ], [ ]
   for name in names:
      if name['kind'] == 'class':
         if 'exceptions' not in name['module']:
            if 'tools' in name['module']:
               tools.append(name)
            elif 'Interface' in name['name'] and not 'Parentage' in name['name']:
               interfaces.append(name)
            elif 'spanner' in name['module']:
               spanners.append(name)
            else:
               klasses.append(name)
         else:
            exceptions.append(name)
      elif name['kind'] == 'function':
         if 'tools' in name['module']:
            tools.append(name)
         else:
            raise ValueError('all public functions must be tools.')
      else:
         raise ValueError('name must be class or function.')
   #print len(klasses)
   #print len(interfaces)
   #print len(spanners)
   #print len(tools)
   #print ''

   result = 'Abjad API\n'
   result += '=' * (len(result) - 1)
   result += '\n\n'
   result += '.. toctree::'
   result += '\n\n'

#   section_title = 'Abjad score components'
#   result += '%s\n' % section_title
#   result += '-' * (len(section_title))
#   result += '\n\n'
#   result += '.. toctree::\n'
#   result += '   :maxdepth: 1\n'
#   result += '\n'
#   for name in klasses:
#      if not name['name'].startswith('_'):
#         doc_path = _module_path_to_doc_path(name['module'])
#         result += '   %s\n' % doc_path
#   result += '\n\n'
  
   # separate autoloading tools packages from manually loading tools packages
   tools.sort(lambda x, y: cmp(x['module'], y['module']))

   manual_loading_tools = [ ]
   manual_loading_tools_names = (
      'configurationtools',
      'durationtools',
      'intervaltreetools', 
      'iotools',
      'layouttools',
      'mathtools',
      'timesignaturetools',
      'pitcharraytools', 
      'sequencetools',
      'sievetools', 
      'tempotools',
      'threadtools',
      'tonalitytools', 
      'verticalitytools',
      )

   unstable_tools = [ ]
   unstable_tools_names = (
      'musicxmltools', 
      'quantizationtools',
   )

   for dictionary in tools[:]:
      for tools_name in manual_loading_tools_names:
         if tools_name in dictionary['module']:
            manual_loading_tools.append(dictionary)
            tools.remove(dictionary)
      for tools_name in unstable_tools_names:
         if tools_name in dictionary['module']:
            unstable_tools.append(dictionary)
            tools.remove(dictionary)   

   section_title = 'Abjad composition packages'
   result += '%s\n' % section_title
   result += '-' * (len(section_title))
   result += '\n\n'
   result += '.. toctree::\n'
   result += '   :maxdepth: 1\n'
   result += '\n'

   last_tools_module = ''
   for name in tools:
      if not name['name'].startswith('_'):
         doc_path = _module_path_to_doc_path(name['module'])
         cur_tools_module = doc_path.split(os.sep)[1]
         if not cur_tools_module == last_tools_module:
            result += '\n\n'
            result += cur_tools_module
            result += '\n\n'
            result += '.. toctree::\n'
            result += '   :maxdepth: 1\n'
            result += '\n'
            last_tools_module = cur_tools_module
         result += '   %s\n' % doc_path
   result += '\n\n'

   section_title = 'Additional Abjad composition packages (load manually)'
   result += '\n%s\n' % section_title
   result += '-' * (len(section_title))
   result += '\n\n'
   result += '.. toctree::\n'
   result += '   :maxdepth: 1\n'
   result += '\n'

   last_tools_module = ''
   for name in manual_loading_tools:
      if not name['name'].startswith('_'):
         doc_path = _module_path_to_doc_path(name['module'])
         cur_tools_module = doc_path.split(os.sep)[1]
         if not cur_tools_module == last_tools_module:
            result += '\n\n'
            result += cur_tools_module
            result += '\n\n'
            result += '.. toctree::\n'
            result += '   :maxdepth: 1\n'
            result += '\n'
            last_tools_module = cur_tools_module
         result += '   %s\n' % doc_path

   result += '\n\n\n'

   section_title = 'Unstable Abjad composition packages (load manually)'
   result += '%s\n' % section_title
   result += '-' * (len(section_title))
   result += '\n\n'
   result += '.. toctree::\n'
   result += '   :maxdepth: 1\n'
   result += '\n'

   last_tools_module = ''
   for name in unstable_tools:
      if not name['name'].startswith('_'):
         doc_path = _module_path_to_doc_path(name['module'])
         cur_tools_module = doc_path.split(os.sep)[1]
         if not cur_tools_module == last_tools_module:
            result += '\n\n'
            result += cur_tools_module
            result += '\n\n'
            result += '.. toctree::\n'
            result += '   :maxdepth: 1\n'
            result += '\n'
            last_tools_module = cur_tools_module
         result += '   %s\n' % doc_path

#   result += '\n\n'
#
#   result += 'Exceptions\n'
#   result += '-' * (len('Exceptions'))
#   result += '\n\n'
#   result += '.. toctree::\n'
#   result += '   :maxdepth: 1\n'
#   result += '\n'
#   result += '   exceptions/exceptions'

   return result



from abjad import *


def test_TempoMark___init___01():
    '''Init tempo mark with integer-valued mark.'''

    t = contexttools.TempoMark(Duration(3, 32), 52)
    assert t.format == '\\tempo 16.=52'


def test_TempoMark___init___02():
    '''Init tempo mark with float-valued mark.'''

    t = contexttools.TempoMark(Duration(3, 32), 52.5)
    assert t.format == '\\tempo 16.=52.5'


def test_TempoMark___init___03():
    '''Init tempo mark from tempo mark.'''

    t = contexttools.TempoMark(Duration(3, 32), 52)
    new = contexttools.TempoMark(t)

    assert t == new
    assert t is not new

    assert t.duration == new.duration
    assert t.duration is not new.duration


def test_TempoMark___init___04():
    '''Init tempo mark from integer pair.'''

    t = contexttools.TempoMark((3, 32), 52.5)
    assert t.format == '\\tempo 16.=52.5'





from abjad.core import _Immutable


class _Segment(tuple, _Immutable):
    '''.. versionadded:: 2.0

    Mix-in base class for ordered collections of pitch objects.
    '''

    ### OVERLOADS ###

    def __add__(self, arg):
        return type(self)(tuple(self) + tuple(arg))

    def __getslice__(self, start, stop):
        return type(self)(tuple.__getslice__(self, start, stop))

    def __mul__(self, n):
        tmp = list(self)
        tmp *= n
        return type(self)(tmp)

    def __rmul__(self, n):
        return self * n




from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals
from abjad import Fraction


def test_intervaltreetools_calculate_sustain_centroid_of_intervals_01():
    tree = IntervalTree(_make_test_intervals())
    result = calculate_sustain_centroid_of_intervals(tree)
    assert result == Fraction(1619, 90)


def test_intervaltreetools_calculate_sustain_centroid_of_intervals_02():
    tree = IntervalTree([])
    result = calculate_sustain_centroid_of_intervals(tree)
    assert result is None



from abjad.tools.intervaltreetools.IntervalTree import IntervalTree
from abjad.tools.intervaltreetools.all_intervals_are_nonoverlapping import all_intervals_are_nonoverlapping
from abjad.tools.intervaltreetools.compute_logical_xor_of_intervals import compute_logical_xor_of_intervals
from abjad.tools.intervaltreetools.mask_intervals_with_intervals import mask_intervals_with_intervals
from collections import Iterable
from abjad import Fraction


def resolve_overlaps_between_nonoverlapping_trees_excluding_remainders_less_than_rational(trees, rational):
    '''Create a nonoverlapping IntervalTree from `trees`.
    Intervals in higher-indexed trees in `trees` only appear in part or whole where they do not
    overlap intervals from lower-indexed trees, and then only where their magnitudes are equal to or greater than
    `rational` ::

        abjad> from abjad.tools import intervaltreetools
        abjad> from abjad.tools.intervaltreetools import BoundedInterval
        abjad> from abjad.tools.intervaltreetools import IntervalTree

    ::

        abjad> a = IntervalTree(BoundedInterval(0, 1, {'a': 1}))
        abjad> b = IntervalTree(BoundedInterval(Fraction(1, 32), Fraction(33, 32), {'b': 2}))
        abjad> c = IntervalTree(BoundedInterval(Fraction(1, 16), Fraction(17, 16), {'c': 3}))
        abjad> intervaltreetools.resolve_overlaps_between_nonoverlapping_trees_excluding_remainders_less_than_rational([a, b, c], Fraction(1, 16))
        IntervalTree([
            BoundedInterval(Offset(0, 1), Offset(1, 1), {'a': 1}),
            BoundedInterval(Offset(1, 1), Offset(17, 16), {'c': 3})
        ])

    Return interval tree.
    '''

    assert isinstance(trees, Iterable) and len(trees) \
        and all([isinstance(x, IntervalTree) for x in trees]) \
        and all([all_intervals_are_nonoverlapping(x) for x in trees])
    assert isinstance(rational, (int, Fraction)) and 0 <= rational

    rtree = trees[0]
    for tree in trees[1:]:
        xor = compute_logical_xor_of_intervals([tree, rtree])
        masked = mask_intervals_with_intervals(tree, xor)
        rtree._insert(filter(lambda x: rational <= x.magnitude, masked))

    assert all_intervals_are_nonoverlapping(rtree)

    return rtree



from abjad.tools.intervaltreetools import *
import py.test


def test_IntervalTree_magnitude_01():
    a = BoundedInterval(-1, 2)
    b = BoundedInterval(0, 1)
    c = BoundedInterval(1, 3)

    tree = IntervalTree(a)
    assert tree.magnitude == 3

    tree = IntervalTree(b)
    assert tree.magnitude == 1

    tree = IntervalTree(c)
    assert tree.magnitude == 2

    tree = IntervalTree([a, b])
    assert tree.magnitude == 3

    tree = IntervalTree([a, c])
    assert tree.magnitude == 4

    tree = IntervalTree([b, c])
    assert tree.magnitude == 3

    tree = IntervalTree([a, b, c])
    assert tree.magnitude == 4



from abjad.tools.musicxmltools._all_are_nodes_with_tag import _all_are_nodes_with_tag
from abjad.tools.musicxmltools._get_staff_id_in_note_node import _get_staff_id_in_note_node


def _get_staff_ids_in_measure_node(node):
    assert _all_are_nodes_with_tag([node], 'measure')
    ids = []
    for note_node in node.findall('note'):
        id = _get_staff_id_in_note_node(note_node)
        if id is not None and id not in ids:
            ids.append(id)
    return tuple(ids)





import os


def _create_subdirectories(root_dir, dirs, interactive):
   '''Create dirs in root_dir.
   If user specifies interactive mode, prompt before each.
   '''

   for dir in dirs:
      new_dir = os.path.join(root_dir, dir)
      if not os.path.exists(new_dir):
         if interactive.lower() in ('y', ''):
            msg = 'Will create new directory "%s". Proceed? [Y/n]: ' % new_dir
            input = raw_input(msg)
         else:
            input = 'Y'
         if input.lower() in ('y', ''):
            print 'Creating directory %s ...' % new_dir
            os.mkdir(new_dir)



from abjad import *
import py.test


def test_marktools_get_lilypond_command_mark_attached_to_component_01():

    note = Note("c'8")
    lilypond_command_mark = marktools.LilyPondCommandMark('stemUp')(note)
    assert marktools.get_lilypond_command_mark_attached_to_component(note) is lilypond_command_mark


def test_marktools_get_lilypond_command_mark_attached_to_component_02():

    note = Note("c'8")
    assert py.test.raises(MissingMarkError, 'marktools.get_lilypond_command_mark_attached_to_component(note)')


def test_marktools_get_lilypond_command_mark_attached_to_component_03():

    note = Note("c'8")
    marktools.LilyPondCommandMark('stemUp')(note)
    marktools.LilyPondCommandMark('slurUp')(note)
    assert py.test.raises(ExtraMarkError, 'marktools.get_lilypond_command_mark_attached_to_component(note)')



from abjad import *


def test_Markup___eq___01():
    '''Markup compare equal when they format the same.
    '''

    markup_1 = markuptools.Markup('foo')
    markup_2 = markuptools.Markup('foo')
    markup_3 = markuptools.Markup('bar')

    assert markup_1 == markup_2
    assert markup_2 == markup_1

    assert not markup_1 == markup_3
    assert not markup_3 == markup_1

    assert not markup_2 == markup_3
    assert not markup_3 == markup_2


def test_Markup___eq___02():
    '''Markup compare equal when they format the same.
    '''

    markup_1 = markuptools.Markup('foo')
    markup_2 = markuptools.Markup('foo', style_string = 'scheme')

    assert not markup_1 == markup_2
    assert not markup_2 == markup_1



from abjad.tools import durationtools
from abjad.tools.contexttools.ContextMark import ContextMark
from abjad.tools import durationtools
import numbers


class TempoMark(ContextMark):
    r'''.. versionadded:: 2.0

    Abjad model of a tempo indication::

        abjad> score = Score([])
        abjad> staff = Staff("c'8 d'8 e'8 f'8")
        abjad> score.append(staff)

    ::

        abjad> contexttools.TempoMark(Duration(1, 8), 52)(staff[0])
        TempoMark(8, 52)(c'8)

    ::

        abjad> f(score)
        \new Score <<
            \tempo 8=52
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
        >>

    Tempo marks target **score** context by default.
    '''

    _format_slot = 'opening'

    def __init__(self, *args, **kwargs):
        from abjad.tools.scoretools.Score import Score
        ContextMark.__init__(self, **kwargs)
        if self.target_context is None:
            self._target_context = Score
        if len(args) == 1 and isinstance(args[0], type(self)):
            tempo_indication = args[0]
            duration = durationtools.Duration(tempo_indication.duration)
            units_per_minute = tempo_indication.units_per_minute
        elif len(args) == 2:
            duration, units_per_minute = args
            #assert isinstance(duration, durationtools.Duration)
            try:
                duration = durationtools.Duration(duration)
            except TypeError:
                duration = durationtools.Duration(*duration)
            assert isinstance(units_per_minute, (int, long, float, durationtools.Duration))
            #duration = duration
            units_per_minute = units_per_minute
        else:
            raise ValueError('can not initialize tempo indication.')
        object.__setattr__(self, '_duration', duration)
        object.__setattr__(self, '_units_per_minute', units_per_minute)

    ### OVERLOADS ###

    def __add__(self, expr):
        if isinstance(expr, type(self)):
            new_quarters_per_minute = self.quarters_per_minute + expr.quarters_per_minute
            minimum_denominator = min((self.duration.denominator, expr.duration.denominator))
            new_units_per_minute, new_duration_denominator = \
                durationtools.rational_to_duration_pair_with_specified_integer_denominator(
                new_quarters_per_minute / 4, minimum_denominator)
            new_duration = durationtools.Duration(1, new_duration_denominator)
            new_tempo_indication = type(self)(new_duration, new_units_per_minute)
            return new_tempo_indication

    def __copy__(self, *args):
        return type(self)(self.duration, self.units_per_minute, target_context = self.target_context)

    def __div__(self, expr):
        if isinstance(expr, type(self)):
            return self.quarters_per_minute / expr.quarters_per_minute
        raise TypeError('must be tempo indication.')

    def __eq__(self, expr):
        if isinstance(expr, type(self)):
            if self.duration == expr.duration:
                if self.units_per_minute == expr.units_per_minute:
                    return True
        return False

    def __mul__(self, multiplier):
        if isinstance(multiplier, (int, float, durationtools.Duration)):
            new_units_per_minute = multiplier * self.units_per_minute
            new_duration = durationtools.Duration(self.duration)
            new_tempo_indication = type(self)(new_duration, new_units_per_minute)
            return new_tempo_indication

    def __sub__(self, expr):
        if isinstance(expr, type(self)):
            new_quarters_per_minute = self.quarters_per_minute - expr.quarters_per_minute
            minimum_denominator = min((self.duration.denominator, expr.duration.denominator))
            new_units_per_minute, new_duration_denominator = \
                durationtools.rational_to_duration_pair_with_specified_integer_denominator(
                new_quarters_per_minute / 4, minimum_denominator)
            new_duration = durationtools.Duration(1, new_duration_denominator)
            new_tempo_indication = type(self)(new_duration, new_units_per_minute)
            return new_tempo_indication

    ### PRIVATE ATTRIBUTES ###

    @property
    def _contents_repr_string(self):
        return '%s, %s' % (self._dotted, self.units_per_minute)

    @property
    def _dotted(self):
        '''Dotted numeral representation of duration.'''
        return durationtools.assignable_rational_to_lilypond_duration_string(self.duration)

    @property
    def _equation(self):
        '''Dotted numeral and units per minute together around equal sign.'''
        return '%s=%s' % (self._dotted, self.units_per_minute)

    ### PUBLIC ATTRIBUTES ###

    @apply
    def duration():
        def fget(self):
            '''Get duration of tempo mark::

                abjad> tempo = contexttools.TempoMark(Duration(1, 8), 52)
                abjad> tempo.duration
                Duration(1, 8)

            Set duration of tempo mark::

                abjad> tempo.duration = Duration(1, 4)
                abjad> tempo.duration
                Duration(1, 4)

            Return duration.
            '''
            return self._duration
        def fset(self, duration):
            try:
                duration = durationtools.Duration(duration)
            except TypeError:
                duration = durationtools.Duration(*duration)
            self._duration = duration
        return property(**locals())

    @property
    def format(self):
        r'''Read-only LilyPond format of tempo mark:

        ::

            abjad> tempo = contexttools.TempoMark(Duration(1, 8), 52)
            abjad> tempo.format
            '\\tempo 8=52'

        Return string.
        '''
        return r'\tempo %s' % self._equation

    @property
    def quarters_per_minute(self):
        r'''Read-only quarters per minute of tempo mark::

            abjad> tempo = contexttools.TempoMark(Duration(1, 8), 52)
            abjad> tempo.quarters_per_minute
            Duration(104, 1)

        Return fraction.
        '''
        return durationtools.Duration(1, 4) / self.duration * self.units_per_minute

    @apply
    def units_per_minute():
        def fget(self):
            r'''Get units per minute of tempo mark::

                abjad> tempo = contexttools.TempoMark(Duration(1, 8), 52)
                abjad> tempo.units_per_minute
                52

            Set units per minute of tempo mark::

                abjad> tempo.units_per_minute = 56
                abjad> tempo.units_per_minute
                56

            Return number.
            '''
            return self._units_per_minute
        def fset(self, units_per_minute):
            assert isinstance(units_per_minute, numbers.Number)
            self._units_per_minute = units_per_minute
        return property(**locals())





from abjad.tools.intervaltreetools.IntervalTree import IntervalTree
from abjad.tools.intervaltreetools.all_are_intervals_or_trees_or_empty import all_are_intervals_or_trees_or_empty


def group_tangent_or_overlapping_intervals_and_yield_groups(intervals):
    '''Group tangent or overlapping intervals in `intervals` and
        return tuples.
    '''

    assert all_are_intervals_or_trees_or_empty(intervals)
    if isinstance(intervals, IntervalTree):
        tree = intervals
    else:
        tree = IntervalTree(intervals)
    if not tree:
        yield IntervalTree([])
        return

    groups = []
    group = [tree[0]]

    low = group[0].low
    high = group[0].high

    for i in range(1, len(tree)):
        if tree[i].low <= high:
            group.append(tree[i])
            if high < tree[i].high:
                high = tree[i].high
        else:
            groups.append(group)
            group = [tree[i]]
            low = group[0].low
            high = group[0].high

    if group not in groups:
        groups.append(group)

    for x in groups:
        yield tuple(x)



from abjad import *
import py.test


def test_Note_written_pitch_indication_is_nonsemantic_01():

    note = Note("c'4")

    assert not note.written_pitch_indication_is_nonsemantic


def test_Note_written_pitch_indication_is_nonsemantic_02():

    note = Note("c'4")
    note.written_pitch_indication_is_nonsemantic = True

    assert note.written_pitch_indication_is_nonsemantic
    assert not note.written_pitch_indication_is_at_sounding_pitch


def test_Note_written_pitch_indication_is_nonsemantic_03():

    note = Note("c'4")

    assert py.test.raises(TypeError, "note.written_pitch_indication_is_nonsemantic = 'foo'")





from abjad.core import _Immutable
from abjad.tools import markuptools
from abjad.tools.tonalitytools.ExtentIndicator import ExtentIndicator
from abjad.tools.tonalitytools.InversionIndicator import InversionIndicator
from abjad.tools.tonalitytools.QualityIndicator import QualityIndicator
from abjad.tools.tonalitytools.ScaleDegree import ScaleDegree
from abjad.tools.tonalitytools.SuspensionIndicator import SuspensionIndicator
import re


class TonalFunction(_Immutable):
    '''.. versionadded:: 2.0

    Abjad model of functions in tonal harmony: I, I6, I64, V, V7, V43, V42,
    bII, bII6, etc., also i, i6, i64, v, v7, etc.

    Value object that can not be cahnged after instantiation.
    '''

    def __init__(self, *args):
        if len(args) == 1 and isinstance(args[0], type(self)):
            scale_degree, quality, extent, inversion, suspension = \
                self._init_by_reference(args[0])
        elif len(args) == 1 and isinstance(args[0], str):
            scale_degree, quality, extent, inversion, suspension = \
                self._init_by_symbolic_string(args[0])
        elif len(args) == 4:
            scale_degree, quality, extent, inversion, suspension = \
                self._init_by_scale_degree_quality_extent_and_inversion(*args)
        elif len(args) == 5:
            scale_degree, quality, extent, inversion, suspension = \
                self._init_with_suspension(*args)
        else:
            raise ValueError('can not initialize tonal function.')
        object.__setattr__(self, '_scale_degree', scale_degree)
        object.__setattr__(self, '_quality', quality)
        object.__setattr__(self, '_extent', extent)
        object.__setattr__(self, '_inversion', inversion)
        object.__setattr__(self, '_suspension', suspension)

    ### OVERLOADS ###

    def __eq__(self, arg):
        if isinstance(arg, type(self)):
            if self.scale_degree == arg.scale_degree:
                if self.quality == arg.quality:
                    if self.extent == arg.extent:
                        if self.inversion == arg.inversion:
                            if self.suspension == arg.suspension:
                                return True
        return False

    def __ne__(self, arg):
        return not self == arg

    def __repr__(self):
        return self._format_string

    ### PRIVATE ATTRIBUTES ###

    @property
    def _accidental_name(self):
        accidental = self.scale_degree.accidental
        if accidental.is_adjusted:
            return accidental.name.title()
        return ''

    @property
    def _figured_bass_digits(self):
        characters = self._figured_bass_string
        if characters:
            characters = characters.split('/')
            digits = [int(x) for x in characters]
            return tuple(digits)
        return ()

    @property
    def _figured_bass_string(self):
        return self.inversion.extent_to_figured_bass_string(self.extent.number)

    _figured_bass_string_to_extent = {
        '': 5, '6': 5, '6/4': 5,
        '7': 7, '6/5': 7, '4/3': 7, '4/2': 7,
    }

    _figured_bass_string_to_inversion = {
        '': 0, '6': 1, '6/4': 2,
        '7': 0, '6/5': 1, '4/3': 2, '4/2': 3,
    }

    @property
    def _format_string(self):
        result = []
        result.append(self._accidental_name)
        result.append(self._roman_numeral_string)
        result.append(self.quality.quality_string.title())
        result.append(self.extent.name.title())
        result.append('In')
        result.append(self.inversion.title)
        if not self.suspension.is_empty:
            result.append('With')
            result.append(self.suspension.title_string)
        return ''.join(result)

    @property
    def _quality_symbolic_string(self):
        if self.extent == ExtentIndicator(5):
            if self.quality == QualityIndicator('diminished'):
                return 'o'
            elif self.quality == QualityIndicator('augmented'):
                return '+'
            else:
                return ''
        elif self.extent == ExtentIndicator(7):
            if self.quality == QualityIndicator('dominant'):
                return ''
            elif self.quality == QualityIndicator('major'):
                return 'M'
            #elif self.quality == QualityIndicator('minor'):
            #   return 'm'
            elif self.quality == QualityIndicator('diminished'):
                return 'o'
            elif self.quality == QualityIndicator('half diminished'):
                return '@'
            elif self.quality == QualityIndicator('augmented'):
                return '+'
            else:
                return ''
        else:
            raise NotImplementedError

    @property
    def _roman_numeral_string(self):
        roman_numeral_string = self.scale_degree.roman_numeral_string
        if not self.quality.is_uppercase:
            roman_numeral_string = roman_numeral_string.lower()
        return roman_numeral_string

    #_symbolic_string_regex = re.compile(
    #    r'([#|b]*)([i|I|v|V]+)([M|m|o|@|+]?)(\d*)')

    #_symbolic_string_regex = re.compile(
    #    r'([#|b]*)([i|I|v|V]+)([M|m|o|@|+]?)(\d*)(\s*)([#|b]?\d*-?[#|b]?\d*)')

    _symbolic_string_regex = re.compile(
        r'([#|b]*)([i|I|v|V]+)([M|m|o|@|+]?)(.*)')

    ### PRIVATE METHODS ###

    def _init_by_reference(self, tonal_function):
        args = (tonal_function.scale_degree, tonal_function.quality,
            tonal_function.extent, tonal_function.inversion)
        return self._init_by_scale_degree_quality_extent_and_inversion(self, *args)

    def _init_by_scale_degree_quality_extent_and_inversion(self, *args):
        scale_degree, quality, extent, inversion = args
        scale_degree = ScaleDegree(scale_degree)
        #self._scale_degree = scale_degree
        quality = QualityIndicator(quality)
        #self._quality = quality
        extent = ExtentIndicator(extent)
        #self._extent = extent
        inversion = InversionIndicator(inversion)
        #self._inversion = inversion
        #self._suspension = SuspensionIndicator()
        suspension = SuspensionIndicator()
        return scale_degree, quality, extent, inversion, suspension

    def _init_by_symbolic_string(self, symbolic_string):
        groups = self._symbolic_string_regex.match(symbolic_string).groups()
        #print groups
        accidental, roman_numeral, quality, figured_bass = groups
        scale_degree = ScaleDegree(accidental + roman_numeral)
        #self._scale_degree = scale_degree
        figured_bass_parts = figured_bass.split('/')
        naive_figured_bass = [x for x in figured_bass_parts if not '-' in x]
        naive_figured_bass = '/'.join(naive_figured_bass)
        extent = self._figured_bass_string_to_extent[naive_figured_bass]
        extent = ExtentIndicator(extent)
        #self._extent = extent
        uppercase = roman_numeral == roman_numeral.upper()
        quality = self._get_quality_name(uppercase, quality, extent.number)
        quality = QualityIndicator(quality)
        #self._quality = quality
        inversion = self._figured_bass_string_to_inversion[naive_figured_bass]
        inversion = InversionIndicator(inversion)
        #self._inversion = inversion
        suspension = [x for x in figured_bass_parts if '-' in x]
        if not suspension:
            suspension = SuspensionIndicator()
        elif 1 < len(suspension):
            raise NotImplementedError('no multiple suspensions yet.')
        else:
            suspension = SuspensionIndicator(suspension[0])
        #self._suspension = suspension
        return scale_degree, quality, extent, inversion, suspension

    def _init_with_suspension(self, *args):
        scale_degree, quality, extent, inversion, suspension = \
            self._init_by_scale_degree_quality_extent_and_inversion(*args[:-1])
        suspension = args[-1]
        #self._suspension = SuspensionIndicator(suspension)
        suspension = SuspensionIndicator(suspension)
        return scale_degree, quality, extent, inversion, suspension

    def _get_quality_name(self, uppercase, quality_string, extent):
        if quality_string == 'o':
            return 'diminished'
        elif quality_string == '@':
            return 'half diminished'
        elif quality_string == '+':
            return 'augmented'
        elif quality_string == 'M':
            return 'major'
        elif quality_string == 'm':
            return 'minor'
        elif extent == 5:
            if quality_string == '' and uppercase:
                return 'major'
            elif quality_string == '' and not uppercase:
                return 'minor'
            else:
                raise ValueError
        elif extent == 7:
            if quality_string == '' and uppercase:
                return 'dominant'
            elif quality_string == '' and not uppercase:
                return 'minor'
            else:
                raise ValueError
        else:
            raise ValueError

    ### PUBLIC ATTRIBUTES ###

    @property
    def bass_scale_degree(self):
        root_scale_degree = self.root_scale_degree.number
        bass_scale_degree = root_scale_degree - 1
        bass_scale_degree += 2 * self.inversion.number
        bass_scale_degree %= 7
        bass_scale_degree += 1
        bass_scale_degree = ScaleDegree(bass_scale_degree)
        return bass_scale_degree

    @property
    def extent(self):
        return self._extent

    @property
    def figured_bass_string(self):
        digits = self._figured_bass_digits
        if self.suspension.is_empty:
            return '/'.join([str(x) for x in digits])
        suspension_pair = self.suspension.figured_bass_pair
        figured_bass_list = []
        for n in range(9, 1, -1):
            if n == suspension_pair[0]:
                figured_bass_list.append(str(self.suspension))
            elif n == suspension_pair[1]:
                pass
            elif n in digits:
                figured_bass_list.append(str(n))
        figured_bass_string = '/'.join(figured_bass_list)
        return figured_bass_string

    @property
    def inversion(self):
        return self._inversion

    @property
    def markup(self):
        symbolic_string = self.symbolic_string
        symbolic_string = symbolic_string.replace('#', r'\sharp ')
        return markuptools.Markup(symbolic_string, 'down')

    @property
    def quality(self):
        return self._quality

    @property
    def root_scale_degree(self):
        return self._scale_degree

    # TODO: deprecate scale_degree in favor of root_scale_degree #
    @property
    def scale_degree(self):
        return self._scale_degree

    @property
    def suspension(self):
        return self._suspension

    @property
    def symbolic_string(self):
        result = ''
        result += self.scale_degree.accidental.symbolic_string
        result += self._roman_numeral_string
        result += self._quality_symbolic_string
        result += self.figured_bass_string
        return result



from abjad.tools import contexttools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools._StringInstrument import _StringInstrument


class Viola(_StringInstrument):
    r'''.. versionadded:: 2.0

    Abjad model of the viola::

        abjad> staff = Staff("c'8 d'8 e'8 f'8")
        abjad> contexttools.ClefMark('alto')(staff)
        ClefMark('alto')(Staff{4})

    ::

        abjad> instrumenttools.Viola()(staff)
        Viola('Viola', 'Va.')(Staff{4})

    ::

        abjad> f(staff)
        \new Staff {
            \clef "alto"
            \set Staff.instrumentName = \markup { Viola }
            \set Staff.shortInstrumentName = \markup { Va. }
            c'8
            d'8
            e'8
            f'8
        }

    The viola targets staff context by default.
    '''

    def __init__(self,
        instrument_name = 'Viola', short_instrument_name = 'Va.', target_context = None):
        _StringInstrument.__init__(self, instrument_name, short_instrument_name, target_context)
        self.sounding_pitch_of_written_middle_c = pitchtools.NamedChromaticPitch("c'")
        self.primary_clefs = [contexttools.ClefMark('alto')]
        self.all_clefs = [
            contexttools.ClefMark('alto'),
            contexttools.ClefMark('treble')]
        self.traditional_range = (-12, 28)



from abjad.tools.componenttools._partition_components_by_durations import _partition_components_by_durations


def partition_components_cyclically_by_durations_in_seconds_le_with_overhang(
    components, durations_in_seconds):
    '''.. versionadded:: 1.1

    Partition `components` cyclically by durations in seconds equal to
    or just less than `durations_in_seconds`, with overhang.
    '''

    parts = _partition_components_by_durations('prolated', components, prolated_duration,
        fill = 'less', cyclic = True, overhang = True)

    return parts



class _StrictComparator(object):
    '''Mix-in base class to confer strict comparison behavior
    to any custom class. Note that this class will be unnecessary
    in some future release because Python 3.0 implements strict
    comparison behavior by default.
    '''

    __slots__ = ( )

    ### OVERLOADS ###

    def __eq__(self, arg):
        return id(self) == id(arg)

    def __ge__(self, arg):
        raise NotImplementedError('\n\tGreater-equal not implemented on "%s".' % repr(arg))

    def __gt__(self, arg):
        raise NotImplementedError('\n\tGreater-than not implemented on "%s".' % repr(arg))

    def __le__(self, arg):
        raise NotImplementedError('\n\tLess-equal not implemented on "%s".' % repr(arg))

    def __lt__(self, arg):
        raise NotImplementedError('\n\tLess-than not implemented on "%s".' % repr(arg))

    def __ne__(self, arg):
        #return id(self) != id(arg)
        return not self == arg

    def __nonzero__(self):
        return True




from abjad.cfg.cfg import ABJADPATH
import os


def make_html_docs():

   # print greeting
   print 'Now building the HTML docs ...'
   print ''

   # change to docs directory because makefile lives there
   docs_directory = os.path.join(ABJADPATH, 'docs')
   os.chdir(docs_directory)

   # make html docs
   os.system('make html')



from abjad.core import _Immutable


class SchemeVector(tuple, _Immutable):
    '''.. versionadded:: 2.0

    Abjad model of Scheme vector::

        abjad> schemetools.SchemeVector(True, True, False)
        SchemeVector(True, True, False)

    Scheme vectors and Scheme vector constants differ in only their LilyPond input format.

    Scheme vectors are immutable.
    '''

    def __new__(klass, *args):
        self = tuple.__new__(klass, args)
        return self

    def __getnewargs__(self):
        return tuple(self)

    ### OVERLOADS ###

    def __repr__(self):
        return '%s(%s)' % (type(self).__name__, self._format_string)

    def __str__(self):
        return '(%s)' % self._output_string

    ### PRIVATE ATTRIBUTES ###

    @property
    def _format_string(self):
        return ', '.join([str(x) for x in self])

    @property
    def _output_string(self):
        vals = []
        for x in self:
            if isinstance(x, type(True)) and x:
                vals.append("#t")
            elif isinstance(x, type(True)):
                vals.append("#f")
            else:
                vals.append(x)
        return ' '.join([str(x) for x in vals])

    ### PUBLIC ATTRIBUTES ###

    @property
    def format(self):
        '''LilyPond input format of Scheme vector::

            abjad> scheme_vector = schemetools.SchemeVector(True, True, False)
            abjad> scheme_vector.format
            "#'(#t #t #f)"

        Return string.
        '''
        return "#'%s" % self.__str__()



from abjad import *


def test_SchemeNumber___eq____01():

    a = schemetools.SchemeNumber(3)
    b = schemetools.SchemeNumber(4)

    assert a != b

def test_SchemeNumber___eq____02():

    a = schemetools.SchemeNumber(3)
    b = schemetools.SchemeNumber(3)

    assert a == b

def test_SchemeNumber___eq____03():

    a = schemetools.SchemeNumber(3)
    b = schemetools.SchemeNumber(3.0)

    assert a == b



from _write_file_interactive import _write_file_interactive
from abjad.cfg.cfg import ABJADPATH
from build_doc_tree import build_doc_tree
from make_sphinx_toc import make_sphinx_toc
import os


def make_sphinx_tocs(interactive = None):
   '''Walk entire Abjad codebase and make Sphinx TOCs automatically.
   '''

   # determine interactivity
   os.system('clear')
   api_doc_path = os.path.join(ABJADPATH, 'docs', 'chapters', 'api')
   modules_visited = [ ]
   if interactive is None:
      interactive = raw_input('Run script in interactive mode? [Y/n]: ')
   elif interactive == True:
      interactive = 'y'
   elif interactive == False:
      interactive = 'n'
   else:
      raise ValueError('unkonwn value for interactive: %s' % interactive)

   # make sphinx tocs
   os.system('clear')
   build_doc_tree(ABJADPATH, api_doc_path, interactive)
   toc = make_sphinx_toc()
   index = os.path.join(api_doc_path, 'index.rst')
   _write_file_interactive(toc, index, interactive)

   # print stop
   print ''
   print '... Done.'
   print ''



from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals


def test_IntervalTree_low_max_01():
    '''low_max returns maximum low value of all intervals in tree.'''
    blocks = _make_test_intervals()
    for i in range(len(blocks)):
        blocks.append(blocks.pop(0))
        tree = IntervalTree(blocks)
        assert tree.low_max == 34

def test_IntervalTree_low_max_02():
    '''low_max returns None if no intervals in tree.'''
    tree = IntervalTree([])
    assert tree.low_max is None




from abjad.tools.intervaltreetools.IntervalTree import IntervalTree
from abjad.tools.intervaltreetools.all_are_intervals_or_trees_or_empty import all_are_intervals_or_trees_or_empty
from abjad import Fraction


def shift_aggregate_offset_to_rational(intervals, rational):
    '''Shift the aggregate offset of `intervals` to `rational` ::

        abjad> from abjad.tools import intervaltreetools
        abjad> from abjad.tools.intervaltreetools import BoundedInterval
        abjad> from abjad.tools.intervaltreetools import IntervalTree

    ::

        abjad> a = BoundedInterval(-1, 3)
        abjad> b = BoundedInterval(6, 12)
        abjad> c = BoundedInterval(9, 16)
        abjad> tree = IntervalTree([a, b, c])
        abjad> intervaltreetools.shift_aggregate_offset_to_rational(tree, Fraction(10, 7))
        IntervalTree([
            BoundedInterval(Offset(10, 7), Offset(38, 7), {}),
            BoundedInterval(Offset(59, 7), Offset(101, 7), {}),
            BoundedInterval(Offset(80, 7), Offset(129, 7), {})
        ])

    Return interval tree.
    '''

    assert isinstance(rational, (int, Fraction))
    assert all_are_intervals_or_trees_or_empty(intervals)
    if isinstance(intervals, IntervalTree):
        tree = intervals
    else:
        tree = IntervalTree(intervals)
    if not tree or rational == tree.low:
        return tree

    shift = rational - tree.low

    return IntervalTree([
        x.shift_by_rational(rational - tree.low) for x in tree
    ])



from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals
import py.test


def test_intervaltreetools_compute_logical_or_of_intervals_in_interval_01():
    a = BoundedInterval(0, 3)
    b = BoundedInterval(6, 12)
    c = BoundedInterval(9, 15)
    tree = IntervalTree([a, b, c])
    d = BoundedInterval(1, 14)
    logic = compute_logical_or_of_intervals_in_interval(tree, d)
    assert [x.signature for x in logic] == [(1, 3), (6, 9), (9, 12), (12, 14)]

def test_intervaltreetools_compute_logical_or_of_intervals_in_interval_02():
    a = BoundedInterval(0, 3)
    b = BoundedInterval(6, 12)
    c = BoundedInterval(9, 15)
    tree = IntervalTree([a, b, c])
    d = BoundedInterval(-1, 16)
    logic = compute_logical_or_of_intervals_in_interval(tree, d)
    assert [x.signature for x in logic] == [(0, 3), (6, 9), (9, 12), (12, 15)]

def test_intervaltreetools_compute_logical_or_of_intervals_in_interval_03():
    a = BoundedInterval(0, 3)
    b = BoundedInterval(6, 12)
    c = BoundedInterval(9, 15)
    tree = IntervalTree([a, b, c])
    d = BoundedInterval(2001, 2010)
    logic = compute_logical_or_of_intervals_in_interval(tree, d)
    assert [x.signature for x in logic] == []




from _write_file_interactive import _write_file_interactive
from make_sphinx_module_listing import make_sphinx_module_listing
import os


def populate_sphinx_directory(root_dir, abjad_subdir, files, interactive):

   # can probably remove modules_visited altogether
   #modules_visited = [ ]

   # make sphinx listing of modules
   for file in files:
      #print file

      #listing = make_sphinx_module_listing('abjad/' + abjad_subdir, file)
      components_path = os.path.join('abjad', abjad_subdir) 
      listing = make_sphinx_module_listing(components_path, file)
      #print listing

      # write no API entry when there is no listing
      if listing is None:
         continue
      
      # write listing to file
      rst_file = file.split('.')[0] + '.rst'
      index = os.path.join(root_dir, rst_file)
      _write_file_interactive(listing, index, interactive)

      # keep track of modules visited
      mod = os.path.join(abjad_subdir, file.split('.')[0])
      #modules_visited.append(mod)



from abjad.tools.containertools.Container import Container
from abjad.tools import componenttools


def get_first_container_in_proper_parentage_of_component(component):
    r'''.. versionadded:: 2.0

    Get first container in proper parentage of `component`::

        abjad> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        abjad> f(staff)
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }

    ::

        abjad> containertools.get_first_container_in_proper_parentage_of_component(staff[1])
        Staff{4}

    Return container or none.
    '''

    return componenttools.get_first_instance_of_klass_in_proper_parentage_of_component(
        component, Container)



from abjad.tools.intervaltreetools.BoundedInterval import BoundedInterval
from abjad import Fraction
import py.test


def test_BoundedInterval_scale_by_rational_01():
    '''BoundedInterval.scale_by_rational returns a new BoundedInterval instance unless offset is 1.'''
    i1 = BoundedInterval(3, 23)
    i2 = i1.scale_by_rational(2)
    assert i1 != i2
    i2 = i1.scale_by_rational(1)
    assert i1 == i2

def test_BoundedInterval_scale_by_rational_02():
    '''BoundedInterval magnitude can be scaled with ints.'''
    i1 = BoundedInterval(3, 23)
    i2 = i1.scale_by_rational(2)
    assert i2.low == 3
    assert i2.magnitude == 40

def test_BoundedInterval_scale_by_rational_03():
    '''BoundedInterval magnitude can be scaled with Fractions.'''
    i1 = BoundedInterval(3, 23)
    i2 = i1.scale_by_rational(Fraction(1, 2))
    assert i2.low == 3
    assert i2.magnitude == 10

def test_BoundedInterval_scale_by_rational_04():
    '''BoundedInterval magnitude cannot be scaled to zero.'''
    i1 = BoundedInterval(3, 23)
    assert py.test.raises(AssertionError,
        'i1.scale_by_rational(0)')

def test_BoundedInterval_scale_by_rational_05():
    '''BoundedInterval magnitude cannot be scaled with negatives.'''
    i1 = BoundedInterval(3, 23)
    assert py.test.raises(AssertionError,
        'i2 = i1.scale_by_rational(-1)')



from abjad import *

__doc__ = '''All Abjad objects should evaluate to True
    unless specifically implemented otherwise.

    You would think that, for example,

    if staff:
        whatever

    would execute for any value of staff.

    But this turns out not be true.
    Python's bool( ) function looks for
    any definition of __nonzero__ on an object;
    if not found, Python evaluates __len__;
    if not found, Python assumes an object is True.

    This is particulary important for containers
    that implement __len__: empty containers carry
    length zero and set bool( ) to False.

    What fixes this is defining __nonzero__ explicitly
    to True on all system objects.'''

def test__StrictComparator___bool___01( ):
    '''Leaves evaluate to True.'''
    assert bool(Note("c'4"))
    assert bool(Rest((1, 4)))
    assert bool(Chord([0, 2, 10], (1, 4)))
    assert bool(skiptools.Skip((1, 4)))


def test__StrictComparator___bool___02( ):
    '''Empty chords evaluate to True,
        even though they carry zero length.'''
    assert bool(Chord([ ], (1, 4)))



def test__StrictComparator___bool___03( ):
    '''Nonempty containers evaluate to True.'''
    assert bool(Staff(Note("c'4") * 4))
    assert bool(Voice(Note("c'4") * 4))
    assert bool(Container(Note("c'4") * 4))
    assert bool(tuplettools.FixedDurationTuplet(Duration(2, 4), Note("c'4") * 3))
    #assert bool(Tuplet(Fraction(2, 3), Note("c'4") * 3))
    assert bool(Tuplet(Fraction(2, 3), Note("c'4") * 3))


def test__StrictComparator___bool___04( ):
    '''Empty containers evaluate to True.'''
    assert bool(Staff([ ]))
    assert bool(Voice([ ]))
    assert bool(Container([ ]))
    assert bool(tuplettools.FixedDurationTuplet(Duration(2, 4), [ ]))
    #assert bool(Tuplet(Fraction(2, 3), [ ]))
    assert bool(Tuplet(Fraction(2, 3), [ ]))




from abjad.tools.stafftools.Staff import Staff


def make_invisible_staff(music):
    '''Staff constructor that hides meter, bar line and staff lines.

    .. versionchanged:: 2.0
        Invisible staff class changed to invisible staff function.
    '''

    staff = Staff(music)
    staff.context = 'RhythmicStaff'
    staff.meter.transparent = True
    staff.bar_line.transparent = True
    staff.staff.transparent = True

    return staff



from abjad.tools.durationtools import Duration
from abjad.tools.intervaltreetools.IntervalTree import IntervalTree
from abjad.tools.intervaltreetools.all_are_intervals_or_trees_or_empty import all_are_intervals_or_trees_or_empty


def calculate_min_mean_and_max_magnitude_of_intervals(intervals):
    '''Return a 3-tuple of the minimum, mean and maximum magnitude of all intervals in `intervals`.
    If `intervals` is empty, return None.
    '''

    assert all_are_intervals_or_trees_or_empty(intervals)
    if isinstance(intervals, IntervalTree):
        tree = intervals
    else:
        tree = IntervalTree(intervals)
    if not tree:
        return None

    magnitudes = [x.magnitude for x in tree]
    return (min(magnitudes), Duration(sum(magnitudes), len(magnitudes)), max(magnitudes))



from abjad.tools import contexttools
from abjad.tools import pitchtools
from abjad.tools import scoretools
from abjad.tools.instrumenttools._KeyboardInstrument import _KeyboardInstrument
from abjad.tools.instrumenttools._ReedInstrument import _ReedInstrument


# TODO: figure out why resolution of effective context as staff group isn't working
class Accordion(_KeyboardInstrument, _ReedInstrument):
    r'''.. versionadded 1.1.2

    Abjad model of the accordion::

        abjad> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        abjad> instrumenttools.Accordion(target_context = Staff)(staff)
        Accordion('Accordion', 'Acc.')(Staff{4})

    ::

        abjad> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { Accordion }
            \set Staff.shortInstrumentName = \markup { Acc. }
            c'8
            d'8
            e'8
            f'8
        }

    The accordion targets piano staff context by default.
    '''

    def __init__( self,
        instrument_name = 'Accordion', short_instrument_name = 'Acc.', target_context = None):
        if target_context is None:
            target_context = scoretools.PianoStaff
        _KeyboardInstrument.__init__(self, instrument_name, short_instrument_name, target_context)
        self.primary_clefs = [contexttools.ClefMark('treble'), contexttools.ClefMark('bass')]
        self._copy_primary_clefs_to_all_clefs()
        self.traditional_range = (-32, 48)



from abjad import Fraction
from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals


def test_intervaltreetools_calculate_depth_centroid_of_intervals_in_interval_01():
    tree = IntervalTree(_make_test_intervals())
    interval = BoundedInterval(0, 10)
    depth_centroid = calculate_depth_centroid_of_intervals_in_interval(tree, interval)
    assert depth_centroid == Fraction(131, 18)


def test_intervaltreetools_calculate_depth_centroid_of_intervals_in_interval_02():
    tree = IntervalTree(_make_test_intervals())
    interval = BoundedInterval(-100, -10)
    depth_centroid = calculate_depth_centroid_of_intervals_in_interval(tree, interval)
    assert depth_centroid is None



from xml.etree import ElementTree
import os


def _import_test_file():
    f = open(os.path.join(os.path.dirname(__file__), 'test', 'SchbAvMaSample.xml'), 'r')
    r = ElementTree.parse(f).getroot()
    f.close()
    return r




from _append_class_options import _append_class_options
from _get_title_type_members import _get_title_type_members
from abjad.cfg.cfg import ABJADPATH
import os


def make_sphinx_module_listing(package_path, file):
   '''This function creates the files like chord.rst.

   Output looks like this:

   Chord
   ==========

   .. automodule:: abjad.components.Chord.chord

   .. autoclass:: abjad.components.Chord
      :members:
      :undoc-members:
      :show-inheritance:
      :inherited-members:

   Returns result as a string.
   '''
   
   #abjad_path = ABJADPATH.rstrip('/abjad')
   abjad_path = os.path.split(ABJADPATH)[0]
   source_full_path = os.path.join(abjad_path, package_path, file)
   file = file.split('.')[0]

   # TODO: tweak me to print accurate but minimal page and sidebar title #
   #package = os.path.join(package_path, file).replace(os.path.sep, '.')

   page_title, auto_type, members = _get_title_type_members(source_full_path)

#   print 'FOO!'
#   print page_title
#   print auto_type
#   print members
#   print 'BAR!'
#   print ''

   # if you want to generate NO api entry for a class of function,
   # then return page_title as None;
   # yes, this is hackish, but it works at least for now.
   if page_title is None:
      return None

   result = '%s\n' %  page_title
   result += '=' * (len(result) - 1)
   result += '\n\n'

   module = os.path.join(package_path, file)
   module = module.replace(os.path.sep, '.')
   #result += '.. automodule:: %s\n' % module
   #result += '\n'

   for member in members:

      # document public helper in tools package like sequencetools.zip_sequences_cyclically
      if auto_type == 'autofunction' and 'tools' in module:
         result += '.. %s:: abjad.tools.%s\n' % (auto_type, page_title)

      # document public class in tools package like pitchtools.NamedChromaticPitchClass
      elif auto_type == 'autoclass' and 'tools' in module:
         result += '.. %s:: abjad.tools.%s\n' % (auto_type, page_title)   
         result = _append_class_options(result)

      # document global public classes like Chord
      elif auto_type == 'autoclass' and not page_title.startswith('_') \
         and not page_title.endswith('Interface'):
         result += '.. %s:: abjad.%s\n' % (auto_type, page_title)   
         result = _append_class_options(result)

      # document public interface like ChordInterface
      elif auto_type == 'autoclass' and not page_title.startswith('_') \
         and (page_title.endswith('Interface') and 'Parentage' not in page_title):
         result += '.. %s:: %s.%s\n' % (auto_type, module, page_title)   
         result = _append_class_options(result)

      # do not document private classes like _ChordFormatter
      elif auto_type == 'autoclass' and page_title.startswith('_'):
         return None

      # do not document parentage interface:
      elif page_title == 'ParentageInterface':
         return None

      # shouldn't be anything else
      else:
         raise ValueError('unknown auto type "%s" or page title "%s".' % (auto_type, page_title))

   return result



from abjad.tools.intervaltreetools.BoundedInterval import BoundedInterval


def test_BoundedInterval_signature_01():
    i = BoundedInterval(0, 10, {'hello': 'world!'})
    assert i.signature == (0, 10)


def test_BoundedInterval_signature_02():
    i = BoundedInterval(-30, 3, {'hellow': 'world!'})
    assert i.signature == (-30, 3)



from abjad.tools.lilyfiletools._BlockAttributed import _BlockAttributed
import types


class PaperBlock(_BlockAttributed):
    r'''.. versionadded:: 2.0

    Abjad model of LilyPond input file paper block.
    '''

    def __init__(self):
        _BlockAttributed.__init__(self)
        self._escaped_name = r'\paper'
        self.minimal_page_breaking = None

    ### PRIVATE ATTRIBUTES ###

    @property
    def _formatted_user_attributes(self):
        result = []
        if self.minimal_page_breaking:
            result.append('#(define page-breaking ly:minimal-breaking)')
        result.extend(_BlockAttributed._formatted_user_attributes.fget(self))
        return result

    ### PUBLIC ATTRIBUTES ###

    @apply
    def minimal_page_breaking():
        def fget(self):
            return self._minimal_page_breaking
        def fset(self, expr):
            if isinstance(expr, (bool, type(None))):
                self._minimal_page_breaking = expr
            else:
                raise TypeError('must be boolean or none')
        return property(**locals())



from abjad.core import _Immutable
from abjad.core import _StrictComparator
from fractions import Fraction


class SchemeMoment(_StrictComparator, _Immutable):
    '''Abjad model of LilyPond moment::

        abjad> schemetools.SchemeMoment(1, 68)
        SchemeMoment(1, 68)

    Initialize scheme moments with a single fraction, two integers or another scheme moment.

    Scheme moments are immutable.
    '''

    __slots__ = ('_duration')

    def __new__(klass, *args):
        self = object.__new__(klass)
        if len(args) == 1 and isinstance(args[0], (Fraction, int, long)):
            object.__setattr__(self, '_duration', Fraction(args[0]))
        elif len(args) == 1 and isinstance(args[0], tuple):
            object.__setattr__(self, '_duration', Fraction(*args[0]))
        elif len(args) == 1 and isinstance(args[0], type(self)):
            object.__setattr__(self, '_duration', args[0].duration)
        elif len(args) == 2 and isinstance(args[0], int) and isinstance(args[1], int):
            object.__setattr__(self, '_duration', Fraction(*args))
        else:
            raise TypeError('can not intialize scheme moment from "%s".' % str(args))
        return self

    def __getnewargs__(self):
        return (self.duration, )

    ### OVERLOADS ###

    def __eq__(self, arg):
        if isinstance(arg, type(self)):
            if self.duration == arg.duration:
                return True
        return False

    def __ge__(self, arg):
        if isinstance(arg, type(self)):
            if self.duration >= arg.duration:
                return True
        return False

    def __gt__(self, arg):
        if isinstance(arg, type(self)):
            if self.duration > arg.duration:
                return True
        return False

    def __le__(self, arg):
        if isinstance(arg, type(self)):
            if self.duration <= arg.duration:
                return True
        return False

    def __lt__(self, arg):
        if isinstance(arg, type(self)):
            if self.duration < arg.duration:
                return True
        return False

    def __ne__(self, arg):
        return not self == arg

    def __repr__(self):
        return '%s(%s, %s)' % (type(self).__name__, self.duration.numerator, self.duration.denominator)

    ### PUBLIC ATTRIBUTES ###

    @property
    def duration(self):
        '''Duration of scheme moment::

            abjad> scheme_moment = schemetools.SchemeMoment(1, 68)
            abjad> scheme_moment.duration
            Fraction(1, 68)

        Return duration.
        '''
        return self._duration

    @property
    def format(self):
        '''LilyPond input format of scheme moment::

            abjad> scheme_moment = schemetools.SchemeMoment(1, 68)
            abjad> scheme_moment.format
            '#(ly:make-moment 1 68)'

        Return string.
        '''
        numerator, denominator = self.duration.numerator, self.duration.denominator
        return '#(ly:make-moment %s %s)' % (numerator, denominator)



from abjad import *


def test_HairpinSpanner___eq___01():
    '''Spanner is strict comparator.
    '''

    spanner_1 = spannertools.HairpinSpanner()
    spanner_2 = spannertools.HairpinSpanner()

    assert not spanner_1 == spanner_2






import os


def _module_path_to_doc_path(module_path):
   '''Transform a module path like

      /Users/foo/abjad/trunk/abjad/tools/sequencetools/zip_cyclic.py

   to a doc path like

      tools/sequencetools/zip_cyclic

   for inclusion in the API TOC.
   '''

   module_parts = module_path.split(os.sep)
   doc_parts = [ ]
   for part in reversed(module_parts):
      if part == 'abjad':
         break
      else:
         doc_parts.insert(0, part)
   doc_path = os.path.join(*doc_parts)

   # remove '.py' extension
   doc_path = doc_path[:-3]

   return doc_path



from abjad.tools.contexttools.get_key_signature_marks_attached_to_component import get_key_signature_marks_attached_to_component


def detach_key_signature_marks_attached_to_component(component):
    r'''.. versionadded:: 2.3

    Detach key signature marks attached to `component`::

        abjad> staff = Staff("c'4 d'4 e'4 f'4")
        abjad> key_signature_mark = contexttools.KeySignatureMark('c', 'major')
        abjad> key_signature_mark.attach(staff)
        KeySignatureMark(NamedChromaticPitchClass('c'), Mode(major))(Staff{4})

    ::

        abjad> f(staff)
        \new Staff {
            \key c \major
            c'4
            d'4
            e'4
            f'4
        }

    ::

        abjad> contexttools.detach_key_signature_marks_attached_to_component(staff)
        (KeySignatureMark(NamedChromaticPitchClass('c'), Mode(major)),)

    ::

        abjad> f(staff)
        \new Staff {
            c'4
            d'4
            e'4
            f'4
        }

    Return tuple of zero or more key signature marks.
    '''

    marks = []
    for mark in get_key_signature_marks_attached_to_component(component):
        mark.detach()
        marks.append(mark)
    return tuple(marks)



from abjad.tools.musicxmltools._all_are_nodes_with_tag import _all_are_nodes_with_tag


def _get_voice_id_in_note_node(node):
    assert _all_are_nodes_with_tag([node], 'note')
    if node.find('voice') is not None:
        return int(node.find('voice').text)
    return None



from abjad.tools.intervaltreetools import BoundedInterval


def test_BoundedInterval___eq___01():
    a = BoundedInterval(0, 10)
    b = BoundedInterval(0, 10)
    assert a == b


def test_BoundedInterval___eq___02():
    a = BoundedInterval(0, 10)
    b = BoundedInterval(-1, 10)
    assert a != b


def test_BoundedInterval___eq___03():
    a = BoundedInterval(0, 10)
    b = BoundedInterval(0, 11)
    assert a != b


def test_BoundedInterval___eq___04():
    a = BoundedInterval(0, 10)
    b = BoundedInterval(0, 10, {})
    assert a == b


def test_BoundedInterval___eq___05():
    a = BoundedInterval(0, 10, {'animal': 'goose'})
    b = BoundedInterval(0, 10, {'animal': 'duck'})
    assert a != b



from abjad import *


def test_TimeSignatureMark___repr___01():
    '''Time signature mark returns nonempty string repr.
    '''

    repr = contexttools.TimeSignatureMark((3, 8)).__repr__()
    assert isinstance(repr, str) and 0 < len(repr)





from abjad.tools.intervaltreetools import *
from abjad.tools.intervaltreetools._make_test_intervals import _make_test_intervals


def test_IntervalTree___iter____01():
    blocks = _make_test_intervals()
    tree = IntervalTree(blocks)
    for e in enumerate(tree):
        assert blocks[e[0]] == e[1]



from abjad import *


def test_MarkupCommand_format_01():

    a = markuptools.MarkupCommand('draw-circle', ['#1', '#0.1', '##f'], None)
    b = markuptools.MarkupCommand('line', None, ['one', 'two', 'three'])
    c = markuptools.MarkupCommand('rounded-box', None, [b])
    d = markuptools.MarkupCommand('combine', None, [a, c], is_braced = False)
    e = markuptools.MarkupCommand('rotate', ['45'], [d])
    f = markuptools.MarkupCommand('triangle', [schemetools.SchemeBoolean(False)], None)
    g = markuptools.MarkupCommand('concat', None, [e, f])

    assert g.format == '\\concat { \\rotate 45 \\combine \\draw-circle #1 #0.1 ##f \\rounded-box \\line { one two three } \\triangle ##f }'
